--
-------------------------------------------------------------------------------------------
-- Copyright  2010-2013, Xilinx, Inc.
-- This file contains confidential and proprietary information of Xilinx, Inc. and is
-- protected under U.S. and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of Xilinx products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
-------------------------------------------------------------------------------------------
--
--
-- Definition of a program memory for KCPSM6 including generic parameters for the 
-- convenient selection of device family, program memory size and the ability to include 
-- the JTAG Loader hardware for rapid software development.
--
-- This file is primarily for use during code development and it is recommended that the 
-- appropriate simplified program memory definition be used in a final production design. 
--
--    Generic                  Values             Comments
--    Parameter                Supported
--  
--    C_FAMILY                 "S6"               Spartan-6 device
--                             "V6"               Virtex-6 device
--                             "7S"               7-Series device 
--                                                  (Artix-7, Kintex-7, Virtex-7 or Zynq)
--
--    C_RAM_SIZE_KWORDS        1, 2 or 4          Size of program memory in K-instructions
--
--    C_JTAG_LOADER_ENABLE     0 or 1             Set to '1' to include JTAG Loader
--
-- Notes
--
-- If your design contains MULTIPLE KCPSM6 instances then only one should have the 
-- JTAG Loader enabled at a time (i.e. make sure that C_JTAG_LOADER_ENABLE is only set to 
-- '1' on one instance of the program memory). Advanced users may be interested to know 
-- that it is possible to connect JTAG Loader to multiple memories and then to use the 
-- JTAG Loader utility to specify which memory contents are to be modified. However, 
-- this scheme does require some effort to set up and the additional connectivity of the 
-- multiple BRAMs can impact the placement, routing and performance of the complete 
-- design. Please contact the author at Xilinx for more detailed information. 
--
-- Regardless of the size of program memory specified by C_RAM_SIZE_KWORDS, the complete 
-- 12-bit address bus is connected to KCPSM6. This enables the generic to be modified 
-- without requiring changes to the fundamental hardware definition. However, when the 
-- program memory is 1K then only the lower 10-bits of the address are actually used and 
-- the valid address range is 000 to 3FF hex. Likewise, for a 2K program only the lower 
-- 11-bits of the address are actually used and the valid address range is 000 to 7FF hex.
--
-- Programs are stored in Block Memory (BRAM) and the number of BRAM used depends on the 
-- size of the program and the device family. 
--
-- In a Spartan-6 device a BRAM is capable of holding 1K instructions. Hence a 2K program 
-- will require 2 BRAMs to be used and a 4K program will require 4 BRAMs to be used. It 
-- should be noted that a 4K program is not such a natural fit in a Spartan-6 device and 
-- the implementation also requires a small amount of logic resulting in slightly lower 
-- performance. A Spartan-6 BRAM can also be split into two 9k-bit memories suggesting 
-- that a program containing up to 512 instructions could be implemented. However, there 
-- is a silicon errata which makes this unsuitable and therefore it is not supported by 
-- this file.
--
-- In a Virtex-6 or any 7-Series device a BRAM is capable of holding 2K instructions so 
-- obviously a 2K program requires only a single BRAM. Each BRAM can also be divided into 
-- 2 smaller memories supporting programs of 1K in half of a 36k-bit BRAM (generally 
-- reported as being an 18k-bit BRAM). For a program of 4K instructions, 2 BRAMs are used.
--
--
-- Program defined by '{psmname}.psm'.
--
-- Generated by KCPSM6 Assembler: 2016-04-19T15:59:36. 
--
-- Assembler used ROM_form template: ROM_form_JTAGLoader_14March13.vhd
--
-- Standard IEEE libraries
--
--
package jtag_loader_pkg is
 function addr_width_calc (size_in_k: integer) return integer;
end jtag_loader_pkg;
--
package body jtag_loader_pkg is
  function addr_width_calc (size_in_k: integer) return integer is
   begin
    if (size_in_k = 1) then return 10;
      elsif (size_in_k = 2) then return 11;
      elsif (size_in_k = 4) then return 12;
      else report "Invalid BlockRAM size. Please set to 1, 2 or 4 K words." severity FAILURE;
    end if;
    return 0;
  end function addr_width_calc;
end package body;
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.jtag_loader_pkg.ALL;
--
-- The Unisim Library is used to define Xilinx primitives. It is also used during
-- simulation. The source can be viewed at %XILINX%\vhdl\src\unisims\unisim_VCOMP.vhd
--  
library unisim;
use unisim.vcomponents.all;
--
--
entity m25p16_spi_uart_bridge is
  generic(             C_FAMILY : string := "S6"; 
              C_RAM_SIZE_KWORDS : integer := 1;
           C_JTAG_LOADER_ENABLE : integer := 0);
  Port (      address : in std_logic_vector(11 downto 0);
          instruction : out std_logic_vector(17 downto 0);
               enable : in std_logic;
                  rdl : out std_logic;                    
                  clk : in std_logic);
  end m25p16_spi_uart_bridge;
--
architecture low_level_definition of m25p16_spi_uart_bridge is
--
signal       address_a : std_logic_vector(15 downto 0);
signal        pipe_a11 : std_logic;
signal       data_in_a : std_logic_vector(35 downto 0);
signal      data_out_a : std_logic_vector(35 downto 0);
signal    data_out_a_l : std_logic_vector(35 downto 0);
signal    data_out_a_h : std_logic_vector(35 downto 0);
signal   data_out_a_ll : std_logic_vector(35 downto 0);
signal   data_out_a_lh : std_logic_vector(35 downto 0);
signal   data_out_a_hl : std_logic_vector(35 downto 0);
signal   data_out_a_hh : std_logic_vector(35 downto 0);
signal       address_b : std_logic_vector(15 downto 0);
signal       data_in_b : std_logic_vector(35 downto 0);
signal     data_in_b_l : std_logic_vector(35 downto 0);
signal    data_in_b_ll : std_logic_vector(35 downto 0);
signal    data_in_b_hl : std_logic_vector(35 downto 0);
signal      data_out_b : std_logic_vector(35 downto 0);
signal    data_out_b_l : std_logic_vector(35 downto 0);
signal   data_out_b_ll : std_logic_vector(35 downto 0);
signal   data_out_b_hl : std_logic_vector(35 downto 0);
signal     data_in_b_h : std_logic_vector(35 downto 0);
signal    data_in_b_lh : std_logic_vector(35 downto 0);
signal    data_in_b_hh : std_logic_vector(35 downto 0);
signal    data_out_b_h : std_logic_vector(35 downto 0);
signal   data_out_b_lh : std_logic_vector(35 downto 0);
signal   data_out_b_hh : std_logic_vector(35 downto 0);
signal        enable_b : std_logic;
signal           clk_b : std_logic;
signal            we_b : std_logic_vector(7 downto 0);
signal          we_b_l : std_logic_vector(3 downto 0);
signal          we_b_h : std_logic_vector(3 downto 0);
-- 
signal       jtag_addr : std_logic_vector(11 downto 0);
signal         jtag_we : std_logic;
signal       jtag_we_l : std_logic;
signal       jtag_we_h : std_logic;
signal        jtag_clk : std_logic;
signal        jtag_din : std_logic_vector(17 downto 0);
signal       jtag_dout : std_logic_vector(17 downto 0);
signal     jtag_dout_1 : std_logic_vector(17 downto 0);
signal         jtag_en : std_logic_vector(0 downto 0);
-- 
signal picoblaze_reset : std_logic_vector(0 downto 0);
signal         rdl_bus : std_logic_vector(0 downto 0);
--
constant BRAM_ADDRESS_WIDTH  : integer := addr_width_calc(C_RAM_SIZE_KWORDS);
--
--
component jtag_loader_6
generic(                C_JTAG_LOADER_ENABLE : integer := 1;
                                    C_FAMILY : string  := "V6";
                             C_NUM_PICOBLAZE : integer := 1;
                       C_BRAM_MAX_ADDR_WIDTH : integer := 10;
          C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                                C_JTAG_CHAIN : integer := 2;
                              C_ADDR_WIDTH_0 : integer := 10;
                              C_ADDR_WIDTH_1 : integer := 10;
                              C_ADDR_WIDTH_2 : integer := 10;
                              C_ADDR_WIDTH_3 : integer := 10;
                              C_ADDR_WIDTH_4 : integer := 10;
                              C_ADDR_WIDTH_5 : integer := 10;
                              C_ADDR_WIDTH_6 : integer := 10;
                              C_ADDR_WIDTH_7 : integer := 10);
port(              picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                           jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                          jtag_din : out STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                         jtag_addr : out STD_LOGIC_VECTOR(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
                          jtag_clk : out std_logic;
                           jtag_we : out std_logic;
                       jtag_dout_0 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_1 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_2 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_3 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_4 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_5 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_6 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_7 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end component;
--
begin
  --
  --  
  ram_1k_generate : if (C_RAM_SIZE_KWORDS = 1) generate
 
    s6: if (C_FAMILY = "S6") generate 
      --
      address_a(13 downto 0) <= address(9 downto 0) & "0000";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "0000000000000000000000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "0000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB16BWER
      generic map ( DATA_WIDTH_A => 18,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 18,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"D920004408200047092000470047129FB001B03171FF003AB6740036B174003A",
                    INIT_01 => X"201E00D70368009E03B203D4601BD040900017001800190720132014F215F820",
                    INIT_02 => X"500095F690001507E0309511900095E9900015B9500035DFD000D57B9000D561",
                    INIT_03 => X"90010000203E108011961298203E1040110D1203203E1010112712005000150A",
                    INIT_04 => X"1200B654500092046049D2019205D208B6645000B67400475000603EB200B100",
                    INIT_05 => X"0620006607200048004E38FB2066004E38FB5000027000480280004802900048",
                    INIT_06 => X"004812040048120000481200B6545000B6740048004E5804191F0064191E5000",
                    INIT_07 => X"004812FF004812FF004812FF0048120100481201004812A8004812C000481280",
                    INIT_08 => X"004812C0004812030048120200481201004812DC004812080048120000481200",
                    INIT_09 => X"1800193950006098C67016010048A2602111B6740048120200481201004812A8",
                    INIT_0A => X"009817131600004812040048120000481200B6546069D280B2005000D2040057",
                    INIT_0B => X"B6740098171C1616004E190CB654012EB615B71419040064B71318081900B674",
                    INIT_0C => X"B2742064003217101901500061101000D1FFB11C0064D701B7261800191D0111",
                    INIT_0D => X"D2140057182819031000D1FF7000611070012064003217081901003AB6740036",
                    INIT_0E => X"20CBD21D20CBD21B20CBD21A20CBD21820D3D21C182C2143D2172133D20020F4",
                    INIT_0F => X"20CFD21C180C2320D21722F9D2000057180819031000D1002138D21320CBD21E",
                    INIT_10 => X"22F2D2152307D21320CBD21020CBD21E20CBD21D20CBD21B20CBD21A20CBD218",
                    INIT_11 => X"18C8006118A80061188800611868006118481700006118280061180817085000",
                    INIT_12 => X"004E5804165017001904710000641701190000CB182800CB1808006118E80061",
                    INIT_13 => X"7D007C007B007A0000641702182C1901203200641701182C1901206602600048",
                    INIT_14 => X"F700D60073006151FFFFDC0470006FF06CC07001005A1828192650007F007E00",
                    INIT_15 => X"004812380048026000480270B654005A182819280B700A601000F700D6002184",
                    INIT_16 => X"C4A02172D3FF2172D20A216DFFFFDC04350014011301E2300048140015001300",
                    INIT_17 => X"DFFF7001006417401901012E182C192827900680B6740950084063306163E5B0",
                    INIT_18 => X"FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF4192",
                    INIT_19 => X"1FFE61A6F40AF50DD302B401B500500013001F014199DF0001DBDF006BA01000",
                    INIT_1A => X"DD8061C002C41ADD1B0269906880677061B060A050001A351B021C005000DD80",
                    INIT_1B => X"1D0021A2E1B51401170161C0C250A54021BFD20003A9190718001740141561BF",
                    INIT_1C => X"900000260020A5A01B0D1A07170050001E0070016990688067706B106A007000",
                    INIT_1D => X"0D2003A9170018001907500061CCCAB01A01475061D190014900480047061004",
                    INIT_1E => X"A1A201C950001A3F1B0217001800190661EED220B205620602C41ACF1B021C00",
                    INIT_1F => X"50001A3F1B021C056200D27350001A3B1B021C0361FAD26521EB1C0261F4D262",
                    INIT_20 => X"50001A101B021C04A1A201C961A202C41AD51B0250001A311B021C0361EBD272",
                    INIT_21 => X"A4E01E005000223F7000FE1FDDFF3E001D0170010032621ED300700063307001",
                    INIT_22 => X"1E001D0003E6221F221FDE0F222DCE301E01700039003800170103C370016240",
                    INIT_23 => X"7001700003E603D47001223F7000170018FE190670014BA01A321B02023F5000",
                    INIT_24 => X"151F1C00170018FF19066D706B806A907001005A18281924225FDC0070006CC0",
                    INIT_25 => X"1924226CDC03226CDC041D006CC067D068B069A002B5182C02A6193470001440",
                    INIT_26 => X"0290700120D3182C02B5182C02A670001440151F70011934226EDC05005A1828",
                    INIT_27 => X"18007001035A700063806280D902227F00CB00D3627FD100180C01806279D901",
                    INIT_28 => X"A28079341800F20B00571903F20A005718081900F109F800039E1300F8005880",
                    INIT_29 => X"0280004802900048026000480270B65470006820226A7000628FD80C18010296",
                    INIT_2A => X"029622B3D20022AFDC0239003800170103A970015000700137001601B6740048",
                    INIT_2B => X"005A19205000D21700571903006417201901012E19245000700062A7B5009401",
                    INIT_2C => X"154722C53B001A0114015000C560A6401000D5004BA01400500062BAF720D600",
                    INIT_2D => X"1574157515411500155F152F152015541553154F15501500152F152015541545",
                    INIT_2E => X"15691573156115421520153A156E156F156915741561157A15691572156F1568",
                    INIT_2F => X"1800191DB6740032B274603202F250007000FE03DDFF3E001D01700115001563",
                    INIT_30 => X"170320320064700067107001180C190102F22064180C1901C064D20117010057",
                    INIT_31 => X"22B5180C02A6191470001C00140015041780180019077001005A180819240064",
                    INIT_32 => X"677066605000D20292059000F710D600005A18081926230FD20100571800191D",
                    INIT_33 => X"6660171016007001004812180048026000480270B65464606570005A18081928",
                    INIT_34 => X"6660655064407001B6746342B7009601D20292046343D2039205D20870006770",
                    INIT_35 => X"12600048120000481200B634206417401901012E180C19282750064070006770",
                    INIT_36 => X"0048120000481201B6340036035A13002399B674004852020230004812000048",
                    INIT_37 => X"127F004812000048120000481203B6340036B674004812200048120C00481200",
                    INIT_38 => X"0048122A004812410048121F0048120900481202B6340036B674004812FF0048",
                    INIT_39 => X"0430B6345000B6740036B6F40036039E131D6399F30ED280B31DB2000036B674",
                    INIT_3A => X"02700047028000470290004712035000B67463A1C3401301E2300048A2301409",
                    INIT_3B => X"130123C1D3FFE230004713000047027000470280004702900047120320440047",
                    INIT_3C => X"03E1004402D00047027000470280004702900047120203E40D205000B67423BB",
                    INIT_3D => X"63DDD20103E1004402700047028000470290004712D803E4500003E663CFD201",
                    INIT_3E => X"0000000000000000000000000000000020441204204412062044004712055000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"88888888888AA822228A222228C2AAB5480808099DDDD8DDAAAC00B56228A6AA",
                   INITP_01 => X"D3760DDDDDDDD37760DDE82AA8274D02A08A020282222D360B58A88888888888",
                   INITP_02 => X"7820581D77755680888A00D7575D78255580A0228018222222220882DDDDDDDD",
                   INITP_03 => X"80D80D8DE800D380202D4D54E80235572D74D8037815580D35421DDB5D5557F7",
                   INITP_04 => X"3B76AD0D388B0CD83745588C0015E0DDFAEC0E0A0ACD756D0BD57B7780EE080D",
                   INITP_05 => X"0ABAC57AAAAAAAAAAAAAAAAAA574D8B58D8822F5B755BB5A2222DBD6128A0A88",
                   INITP_06 => X"2222A88888A8A8222228205D57B58C2D438889605C35836088C003822B70A0D2",
                   INITP_07 => X"00000000000088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA2")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a(31 downto 0),
                  DOPA => data_out_a(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b(31 downto 0),
                  DOPB => data_out_b(35 downto 32), 
                   DIB => data_in_b(31 downto 0),
                  DIPB => data_in_b(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --               
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"D920004408200047092000470047129FB001B03171FF003AB6740036B174003A",
                    INIT_01 => X"201E00D70368009E03B203D4601BD040900017001800190720132014F215F820",
                    INIT_02 => X"500095F690001507E0309511900095E9900015B9500035DFD000D57B9000D561",
                    INIT_03 => X"90010000203E108011961298203E1040110D1203203E1010112712005000150A",
                    INIT_04 => X"1200B654500092046049D2019205D208B6645000B67400475000603EB200B100",
                    INIT_05 => X"0620006607200048004E38FB2066004E38FB5000027000480280004802900048",
                    INIT_06 => X"004812040048120000481200B6545000B6740048004E5804191F0064191E5000",
                    INIT_07 => X"004812FF004812FF004812FF0048120100481201004812A8004812C000481280",
                    INIT_08 => X"004812C0004812030048120200481201004812DC004812080048120000481200",
                    INIT_09 => X"1800193950006098C67016010048A2602111B6740048120200481201004812A8",
                    INIT_0A => X"009817131600004812040048120000481200B6546069D280B2005000D2040057",
                    INIT_0B => X"B6740098171C1616004E190CB654012EB615B71419040064B71318081900B674",
                    INIT_0C => X"B2742064003217101901500061101000D1FFB11C0064D701B7261800191D0111",
                    INIT_0D => X"D2140057182819031000D1FF7000611070012064003217081901003AB6740036",
                    INIT_0E => X"20CBD21D20CBD21B20CBD21A20CBD21820D3D21C182C2143D2172133D20020F4",
                    INIT_0F => X"20CFD21C180C2320D21722F9D2000057180819031000D1002138D21320CBD21E",
                    INIT_10 => X"22F2D2152307D21320CBD21020CBD21E20CBD21D20CBD21B20CBD21A20CBD218",
                    INIT_11 => X"18C8006118A80061188800611868006118481700006118280061180817085000",
                    INIT_12 => X"004E5804165017001904710000641701190000CB182800CB1808006118E80061",
                    INIT_13 => X"7D007C007B007A0000641702182C1901203200641701182C1901206602600048",
                    INIT_14 => X"F700D60073006151FFFFDC0470006FF06CC07001005A1828192650007F007E00",
                    INIT_15 => X"004812380048026000480270B654005A182819280B700A601000F700D6002184",
                    INIT_16 => X"C4A02172D3FF2172D20A216DFFFFDC04350014011301E2300048140015001300",
                    INIT_17 => X"DFFF7001006417401901012E182C192827900680B6740950084063306163E5B0",
                    INIT_18 => X"FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF4192",
                    INIT_19 => X"1FFE61A6F40AF50DD302B401B500500013001F014199DF0001DBDF006BA01000",
                    INIT_1A => X"DD8061C002C41ADD1B0269906880677061B060A050001A351B021C005000DD80",
                    INIT_1B => X"1D0021A2E1B51401170161C0C250A54021BFD20003A9190718001740141561BF",
                    INIT_1C => X"900000260020A5A01B0D1A07170050001E0070016990688067706B106A007000",
                    INIT_1D => X"0D2003A9170018001907500061CCCAB01A01475061D190014900480047061004",
                    INIT_1E => X"A1A201C950001A3F1B0217001800190661EED220B205620602C41ACF1B021C00",
                    INIT_1F => X"50001A3F1B021C056200D27350001A3B1B021C0361FAD26521EB1C0261F4D262",
                    INIT_20 => X"50001A101B021C04A1A201C961A202C41AD51B0250001A311B021C0361EBD272",
                    INIT_21 => X"A4E01E005000223F7000FE1FDDFF3E001D0170010032621ED300700063307001",
                    INIT_22 => X"1E001D0003E6221F221FDE0F222DCE301E01700039003800170103C370016240",
                    INIT_23 => X"7001700003E603D47001223F7000170018FE190670014BA01A321B02023F5000",
                    INIT_24 => X"151F1C00170018FF19066D706B806A907001005A18281924225FDC0070006CC0",
                    INIT_25 => X"1924226CDC03226CDC041D006CC067D068B069A002B5182C02A6193470001440",
                    INIT_26 => X"0290700120D3182C02B5182C02A670001440151F70011934226EDC05005A1828",
                    INIT_27 => X"18007001035A700063806280D902227F00CB00D3627FD100180C01806279D901",
                    INIT_28 => X"A28079341800F20B00571903F20A005718081900F109F800039E1300F8005880",
                    INIT_29 => X"0280004802900048026000480270B65470006820226A7000628FD80C18010296",
                    INIT_2A => X"029622B3D20022AFDC0239003800170103A970015000700137001601B6740048",
                    INIT_2B => X"005A19205000D21700571903006417201901012E19245000700062A7B5009401",
                    INIT_2C => X"154722C53B001A0114015000C560A6401000D5004BA01400500062BAF720D600",
                    INIT_2D => X"1574157515411500155F152F152015541553154F15501500152F152015541545",
                    INIT_2E => X"15691573156115421520153A156E156F156915741561157A15691572156F1568",
                    INIT_2F => X"1800191DB6740032B274603202F250007000FE03DDFF3E001D01700115001563",
                    INIT_30 => X"170320320064700067107001180C190102F22064180C1901C064D20117010057",
                    INIT_31 => X"22B5180C02A6191470001C00140015041780180019077001005A180819240064",
                    INIT_32 => X"677066605000D20292059000F710D600005A18081926230FD20100571800191D",
                    INIT_33 => X"6660171016007001004812180048026000480270B65464606570005A18081928",
                    INIT_34 => X"6660655064407001B6746342B7009601D20292046343D2039205D20870006770",
                    INIT_35 => X"12600048120000481200B634206417401901012E180C19282750064070006770",
                    INIT_36 => X"0048120000481201B6340036035A13002399B674004852020230004812000048",
                    INIT_37 => X"127F004812000048120000481203B6340036B674004812200048120C00481200",
                    INIT_38 => X"0048122A004812410048121F0048120900481202B6340036B674004812FF0048",
                    INIT_39 => X"0430B6345000B6740036B6F40036039E131D6399F30ED280B31DB2000036B674",
                    INIT_3A => X"02700047028000470290004712035000B67463A1C3401301E2300048A2301409",
                    INIT_3B => X"130123C1D3FFE230004713000047027000470280004702900047120320440047",
                    INIT_3C => X"03E1004402D00047027000470280004702900047120203E40D205000B67423BB",
                    INIT_3D => X"63DDD20103E1004402700047028000470290004712D803E4500003E663CFD201",
                    INIT_3E => X"0000000000000000000000000000000020441204204412062044004712055000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"88888888888AA822228A222228C2AAB5480808099DDDD8DDAAAC00B56228A6AA",
                   INITP_01 => X"D3760DDDDDDDD37760DDE82AA8274D02A08A020282222D360B58A88888888888",
                   INITP_02 => X"7820581D77755680888A00D7575D78255580A0228018222222220882DDDDDDDD",
                   INITP_03 => X"80D80D8DE800D380202D4D54E80235572D74D8037815580D35421DDB5D5557F7",
                   INITP_04 => X"3B76AD0D388B0CD83745588C0015E0DDFAEC0E0A0ACD756D0BD57B7780EE080D",
                   INITP_05 => X"0ABAC57AAAAAAAAAAAAAAAAAA574D8B58D8822F5B755BB5A2222DBD6128A0A88",
                   INITP_06 => X"2222A88888A8A8222228205D57B58C2D438889605C35836088C003822B70A0D2",
                   INITP_07 => X"00000000000088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA2")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"D920004408200047092000470047129FB001B03171FF003AB6740036B174003A",
                    INIT_01 => X"201E00D70368009E03B203D4601BD040900017001800190720132014F215F820",
                    INIT_02 => X"500095F690001507E0309511900095E9900015B9500035DFD000D57B9000D561",
                    INIT_03 => X"90010000203E108011961298203E1040110D1203203E1010112712005000150A",
                    INIT_04 => X"1200B654500092046049D2019205D208B6645000B67400475000603EB200B100",
                    INIT_05 => X"0620006607200048004E38FB2066004E38FB5000027000480280004802900048",
                    INIT_06 => X"004812040048120000481200B6545000B6740048004E5804191F0064191E5000",
                    INIT_07 => X"004812FF004812FF004812FF0048120100481201004812A8004812C000481280",
                    INIT_08 => X"004812C0004812030048120200481201004812DC004812080048120000481200",
                    INIT_09 => X"1800193950006098C67016010048A2602111B6740048120200481201004812A8",
                    INIT_0A => X"009817131600004812040048120000481200B6546069D280B2005000D2040057",
                    INIT_0B => X"B6740098171C1616004E190CB654012EB615B71419040064B71318081900B674",
                    INIT_0C => X"B2742064003217101901500061101000D1FFB11C0064D701B7261800191D0111",
                    INIT_0D => X"D2140057182819031000D1FF7000611070012064003217081901003AB6740036",
                    INIT_0E => X"20CBD21D20CBD21B20CBD21A20CBD21820D3D21C182C2143D2172133D20020F4",
                    INIT_0F => X"20CFD21C180C2320D21722F9D2000057180819031000D1002138D21320CBD21E",
                    INIT_10 => X"22F2D2152307D21320CBD21020CBD21E20CBD21D20CBD21B20CBD21A20CBD218",
                    INIT_11 => X"18C8006118A80061188800611868006118481700006118280061180817085000",
                    INIT_12 => X"004E5804165017001904710000641701190000CB182800CB1808006118E80061",
                    INIT_13 => X"7D007C007B007A0000641702182C1901203200641701182C1901206602600048",
                    INIT_14 => X"F700D60073006151FFFFDC0470006FF06CC07001005A1828192650007F007E00",
                    INIT_15 => X"004812380048026000480270B654005A182819280B700A601000F700D6002184",
                    INIT_16 => X"C4A02172D3FF2172D20A216DFFFFDC04350014011301E2300048140015001300",
                    INIT_17 => X"DFFF7001006417401901012E182C192827900680B6740950084063306163E5B0",
                    INIT_18 => X"FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF4192",
                    INIT_19 => X"1FFE61A6F40AF50DD302B401B500500013001F014199DF0001DBDF006BA01000",
                    INIT_1A => X"DD8061C002C41ADD1B0269906880677061B060A050001A351B021C005000DD80",
                    INIT_1B => X"1D0021A2E1B51401170161C0C250A54021BFD20003A9190718001740141561BF",
                    INIT_1C => X"900000260020A5A01B0D1A07170050001E0070016990688067706B106A007000",
                    INIT_1D => X"0D2003A9170018001907500061CCCAB01A01475061D190014900480047061004",
                    INIT_1E => X"A1A201C950001A3F1B0217001800190661EED220B205620602C41ACF1B021C00",
                    INIT_1F => X"50001A3F1B021C056200D27350001A3B1B021C0361FAD26521EB1C0261F4D262",
                    INIT_20 => X"50001A101B021C04A1A201C961A202C41AD51B0250001A311B021C0361EBD272",
                    INIT_21 => X"A4E01E005000223F7000FE1FDDFF3E001D0170010032621ED300700063307001",
                    INIT_22 => X"1E001D0003E6221F221FDE0F222DCE301E01700039003800170103C370016240",
                    INIT_23 => X"7001700003E603D47001223F7000170018FE190670014BA01A321B02023F5000",
                    INIT_24 => X"151F1C00170018FF19066D706B806A907001005A18281924225FDC0070006CC0",
                    INIT_25 => X"1924226CDC03226CDC041D006CC067D068B069A002B5182C02A6193470001440",
                    INIT_26 => X"0290700120D3182C02B5182C02A670001440151F70011934226EDC05005A1828",
                    INIT_27 => X"18007001035A700063806280D902227F00CB00D3627FD100180C01806279D901",
                    INIT_28 => X"A28079341800F20B00571903F20A005718081900F109F800039E1300F8005880",
                    INIT_29 => X"0280004802900048026000480270B65470006820226A7000628FD80C18010296",
                    INIT_2A => X"029622B3D20022AFDC0239003800170103A970015000700137001601B6740048",
                    INIT_2B => X"005A19205000D21700571903006417201901012E19245000700062A7B5009401",
                    INIT_2C => X"154722C53B001A0114015000C560A6401000D5004BA01400500062BAF720D600",
                    INIT_2D => X"1574157515411500155F152F152015541553154F15501500152F152015541545",
                    INIT_2E => X"15691573156115421520153A156E156F156915741561157A15691572156F1568",
                    INIT_2F => X"1800191DB6740032B274603202F250007000FE03DDFF3E001D01700115001563",
                    INIT_30 => X"170320320064700067107001180C190102F22064180C1901C064D20117010057",
                    INIT_31 => X"22B5180C02A6191470001C00140015041780180019077001005A180819240064",
                    INIT_32 => X"677066605000D20292059000F710D600005A18081926230FD20100571800191D",
                    INIT_33 => X"6660171016007001004812180048026000480270B65464606570005A18081928",
                    INIT_34 => X"6660655064407001B6746342B7009601D20292046343D2039205D20870006770",
                    INIT_35 => X"12600048120000481200B634206417401901012E180C19282750064070006770",
                    INIT_36 => X"0048120000481201B6340036035A13002399B674004852020230004812000048",
                    INIT_37 => X"127F004812000048120000481203B6340036B674004812200048120C00481200",
                    INIT_38 => X"0048122A004812410048121F0048120900481202B6340036B674004812FF0048",
                    INIT_39 => X"0430B6345000B6740036B6F40036039E131D6399F30ED280B31DB2000036B674",
                    INIT_3A => X"02700047028000470290004712035000B67463A1C3401301E2300048A2301409",
                    INIT_3B => X"130123C1D3FFE230004713000047027000470280004702900047120320440047",
                    INIT_3C => X"03E1004402D00047027000470280004702900047120203E40D205000B67423BB",
                    INIT_3D => X"63DDD20103E1004402700047028000470290004712D803E4500003E663CFD201",
                    INIT_3E => X"0000000000000000000000000000000020441204204412062044004712055000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"88888888888AA822228A222228C2AAB5480808099DDDD8DDAAAC00B56228A6AA",
                   INITP_01 => X"D3760DDDDDDDD37760DDE82AA8274D02A08A020282222D360B58A88888888888",
                   INITP_02 => X"7820581D77755680888A00D7575D78255580A0228018222222220882DDDDDDDD",
                   INITP_03 => X"80D80D8DE800D380202D4D54E80235572D74D8037815580D35421DDB5D5557F7",
                   INITP_04 => X"3B76AD0D388B0CD83745588C0015E0DDFAEC0E0A0ACD756D0BD57B7780EE080D",
                   INITP_05 => X"0ABAC57AAAAAAAAAAAAAAAAAA574D8B58D8822F5B755BB5A2222DBD6128A0A88",
                   INITP_06 => X"2222A88888A8A8222228205D57B58C2D438889605C35836088C003822B70A0D2",
                   INITP_07 => X"00000000000088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA2")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate akv7;
    --
  end generate ram_1k_generate;
  --
  --
  --
  ram_2k_generate : if (C_RAM_SIZE_KWORDS = 2) generate
    --
    --
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"1ED7689EB2D41B400000000713141520204420472047479F0131FF3A7436743A",
                    INIT_01 => X"01003E8096983E400D033E102700000A00F60007301100E900B900DF007B0061",
                    INIT_02 => X"206620484EFB664EFB00704880489048005400044901050864007447003E0000",
                    INIT_03 => X"48FF48FF48FF4801480148A848C04880480448004800540074484E041F641E00",
                    INIT_04 => X"003900987001486011744802480148A848C048034802480148DC480848004800",
                    INIT_05 => X"74981C164E0C542E151404641308007498130048044800480054698000000457",
                    INIT_06 => X"1457280300FF001001643208013A74367464321001001000FF1C640126001D11",
                    INIT_07 => X"CF1C0C2017F90057080300003813CB1ECB1DCB1BCB1ACB18D31C2C43173300F4",
                    INIT_08 => X"C861A861886168614800612861080800F2150713CB10CB1ECB1DCB1BCB1ACB18",
                    INIT_09 => X"0000000064022C013264012C016660484E0450000400640100CB28CB0861E861",
                    INIT_0A => X"483848604870545A282870600000008400000051FF0400F0C0015A2826000000",
                    INIT_0B => X"FF016440012E2C2890807450403063B0A072FF720A6DFF040001013048000000",
                    INIT_0C => X"FEA60A0D0201000000019900DB00A000000000F0C0B0A090807030010000FF92",
                    INIT_0D => X"00A2B50101C05040BF00A907004015BF80C0C4DD02908070B0A0003502000080",
                    INIT_0E => X"20A900000700CCB00150D10100000604002620A00D0700000001908070100000",
                    INIT_0F => X"003F02050073003B0203FA65EB02F462A2C9003F02000006EE200506C4CF0200",
                    INIT_10 => X"E000003F001FFF000101321E0000300100100204A2C9A2C4D50200310203EB72",
                    INIT_11 => X"0100E6D4013F0000FE0601A032023F000000E61F1F0F2D300100000001C30140",
                    INIT_12 => X"246C036C0400C0D0B0A0B52CA63400401F0000FF06708090015A28245F0000C0",
                    INIT_13 => X"00015A008080027FCBD37F000C8079019001D32CB52CA600401F01346E055A28",
                    INIT_14 => X"804890486048705400206A008F0C01968034000B57030A57080009009E000080",
                    INIT_15 => X"5A20001757036420012E240000A7000196B300AF02000001A901000100017448",
                    INIT_16 => X"747541005F2F2054534F50002F20544547C50001010060400000A00000BA2000",
                    INIT_17 => X"001D74327432F2000003FF000101006369736142203A6E6F6974617A69726F68",
                    INIT_18 => X"B50CA61400000004800007015A0824640332640010010C01F2640C0164010157",
                    INIT_19 => X"601000014818486048705460705A082870600002050010005A08260F0157001D",
                    INIT_1A => X"6048004800346440012E0C285040007060504001744200010204430305080070",
                    INIT_1B => X"7F4800480048033436744820480C48004800480134365A009974480230480048",
                    INIT_1C => X"3034007436F4369E1D990E801D003674482A4841481F4809480234367448FF48",
                    INIT_1D => X"01C1FF30470047704780479047034447704780479047030074A1400130483009",
                    INIT_1E => X"DD01E144704780479047D8E400E6CF01E144D04770478047904702E4200074BB",
                    INIT_1F => X"0000000000000000000000000000000000000000000000004404440644470500",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"1058001415181ADB256A4000408000000000000A20000001088955552C508822",
                   INITP_01 => X"20AACD82EA6A1A2CEDB5CD89DAFE95639D9776A30065B90AA5281D8000422000",
                   INITP_02 => X"402BFFFFFFFFA26245E205880000446C2A1500508544AE103154602C028A2E4A",
                   INITP_03 => X"00000000A0168019E400007001E800000000038001D946214009823111A28912",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_l(31 downto 0),
                  DOPA => data_out_a_l(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_l(31 downto 0),
                  DOPB => data_out_b_l(35 downto 32), 
                   DIB => data_in_b_l(31 downto 0),
                  DIPB => data_in_b_l(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_h: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"100001000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"C800100808091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA",
                    INIT_02 => X"03000300001C10001C28010001000100095B2849B06949695B285B0028B0D9D8",
                    INIT_03 => X"000900090009000900090009000900090009000900095B285B00002C0C000C28",
                    INIT_04 => X"0C0C28B0E38B0051105B00090009000900090009000900090009000900090009",
                    INIT_05 => X"5B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E900",
                    INIT_06 => X"E9000C0C88E8B8B0B810000B0C005B005910000B0C28B088E85880EB5B0C0C00",
                    INIT_07 => X"90E90C91E991E9000C0C88E890E990E990E990E990E990E990E90C90E990E990",
                    INIT_08 => X"0C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E990E9",
                    INIT_09 => X"BEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C00",
                    INIT_0A => X"0009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BFBF",
                    INIT_0B => X"EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A09",
                    INIT_0C => X"0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EFA0",
                    INIT_0D => X"0E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8EE",
                    INIT_0E => X"06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5B8",
                    INIT_0F => X"280D0D0EB1E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D0E",
                    INIT_10 => X"520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0E9",
                    INIT_11 => X"B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8B1",
                    INIT_12 => X"0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8B6",
                    INIT_13 => X"0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE000C",
                    INIT_14 => X"010001000100015BB8B411B8B1EC8C0151BC0C79000C79000C0C787C01097C2C",
                    INIT_15 => X"000CA8E9000C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B00",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB",
                    INIT_17 => X"0C0C5B0059B00128B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"110C010CB80E0A0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B00",
                    INIT_19 => X"B30B0BB80009000100015BB2B2000C0CB3B3A86949C8FBEB000C0C91E9000C0C",
                    INIT_1A => X"09000900095B100B0C000C0C9383B8B3B3B2B2B85BB1DBCB6949B1694969B8B3",
                    INIT_1B => X"090009000900095B005B000900090009000900095B000109115B002901000900",
                    INIT_1C => X"025B285B005B000109B1F9E95959005B000900090009000900095B005B000900",
                    INIT_1D => X"8991E97100090001000100010009100001000100010009285BB1E1897100518A",
                    INIT_1E => X"B169010001000100010009012801B16901000100010001000100090106285B11",
                    INIT_1F => X"0000000000000000000000000000000000000000000000001009100910000928",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"952AAA954AE7E521CB11956532EAAAAAAAAAABE55B5569FC2222AAAAFE0C56DF",
                   INITP_01 => X"8A2AE0984620E14164A16022412B201D64225418AB09126408C582555529AAAA",
                   INITP_02 => X"3F87FFFFFFFFC4ACAA5CD0F355B91B3A75E26B2A502A00CAFE333A4638758F22",
                   INITP_03 => X"000000ADB55ED5575AAB55CC7F43AABD55EAAEE556421CA61AA42494A81974C9",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_h(31 downto 0),
                  DOPA => data_out_a_h(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_h(31 downto 0),
                  DOPB => data_out_b_h(35 downto 32), 
                   DIB => data_in_b_h(31 downto 0),
                  DIPB => data_in_b_h(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"D920004408200047092000470047129FB001B03171FF003AB6740036B174003A",
                    INIT_01 => X"201E00D70368009E03B203D4601BD040900017001800190720132014F215F820",
                    INIT_02 => X"500095F690001507E0309511900095E9900015B9500035DFD000D57B9000D561",
                    INIT_03 => X"90010000203E108011961298203E1040110D1203203E1010112712005000150A",
                    INIT_04 => X"1200B654500092046049D2019205D208B6645000B67400475000603EB200B100",
                    INIT_05 => X"0620006607200048004E38FB2066004E38FB5000027000480280004802900048",
                    INIT_06 => X"004812040048120000481200B6545000B6740048004E5804191F0064191E5000",
                    INIT_07 => X"004812FF004812FF004812FF0048120100481201004812A8004812C000481280",
                    INIT_08 => X"004812C0004812030048120200481201004812DC004812080048120000481200",
                    INIT_09 => X"1800193950006098C67016010048A2602111B6740048120200481201004812A8",
                    INIT_0A => X"009817131600004812040048120000481200B6546069D280B2005000D2040057",
                    INIT_0B => X"B6740098171C1616004E190CB654012EB615B71419040064B71318081900B674",
                    INIT_0C => X"B2742064003217101901500061101000D1FFB11C0064D701B7261800191D0111",
                    INIT_0D => X"D2140057182819031000D1FF7000611070012064003217081901003AB6740036",
                    INIT_0E => X"20CBD21D20CBD21B20CBD21A20CBD21820D3D21C182C2143D2172133D20020F4",
                    INIT_0F => X"20CFD21C180C2320D21722F9D2000057180819031000D1002138D21320CBD21E",
                    INIT_10 => X"22F2D2152307D21320CBD21020CBD21E20CBD21D20CBD21B20CBD21A20CBD218",
                    INIT_11 => X"18C8006118A80061188800611868006118481700006118280061180817085000",
                    INIT_12 => X"004E5804165017001904710000641701190000CB182800CB1808006118E80061",
                    INIT_13 => X"7D007C007B007A0000641702182C1901203200641701182C1901206602600048",
                    INIT_14 => X"F700D60073006151FFFFDC0470006FF06CC07001005A1828192650007F007E00",
                    INIT_15 => X"004812380048026000480270B654005A182819280B700A601000F700D6002184",
                    INIT_16 => X"C4A02172D3FF2172D20A216DFFFFDC04350014011301E2300048140015001300",
                    INIT_17 => X"DFFF7001006417401901012E182C192827900680B6740950084063306163E5B0",
                    INIT_18 => X"FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF4192",
                    INIT_19 => X"1FFE61A6F40AF50DD302B401B500500013001F014199DF0001DBDF006BA01000",
                    INIT_1A => X"DD8061C002C41ADD1B0269906880677061B060A050001A351B021C005000DD80",
                    INIT_1B => X"1D0021A2E1B51401170161C0C250A54021BFD20003A9190718001740141561BF",
                    INIT_1C => X"900000260020A5A01B0D1A07170050001E0070016990688067706B106A007000",
                    INIT_1D => X"0D2003A9170018001907500061CCCAB01A01475061D190014900480047061004",
                    INIT_1E => X"A1A201C950001A3F1B0217001800190661EED220B205620602C41ACF1B021C00",
                    INIT_1F => X"50001A3F1B021C056200D27350001A3B1B021C0361FAD26521EB1C0261F4D262",
                    INIT_20 => X"50001A101B021C04A1A201C961A202C41AD51B0250001A311B021C0361EBD272",
                    INIT_21 => X"A4E01E005000223F7000FE1FDDFF3E001D0170010032621ED300700063307001",
                    INIT_22 => X"1E001D0003E6221F221FDE0F222DCE301E01700039003800170103C370016240",
                    INIT_23 => X"7001700003E603D47001223F7000170018FE190670014BA01A321B02023F5000",
                    INIT_24 => X"151F1C00170018FF19066D706B806A907001005A18281924225FDC0070006CC0",
                    INIT_25 => X"1924226CDC03226CDC041D006CC067D068B069A002B5182C02A6193470001440",
                    INIT_26 => X"0290700120D3182C02B5182C02A670001440151F70011934226EDC05005A1828",
                    INIT_27 => X"18007001035A700063806280D902227F00CB00D3627FD100180C01806279D901",
                    INIT_28 => X"A28079341800F20B00571903F20A005718081900F109F800039E1300F8005880",
                    INIT_29 => X"0280004802900048026000480270B65470006820226A7000628FD80C18010296",
                    INIT_2A => X"029622B3D20022AFDC0239003800170103A970015000700137001601B6740048",
                    INIT_2B => X"005A19205000D21700571903006417201901012E19245000700062A7B5009401",
                    INIT_2C => X"154722C53B001A0114015000C560A6401000D5004BA01400500062BAF720D600",
                    INIT_2D => X"1574157515411500155F152F152015541553154F15501500152F152015541545",
                    INIT_2E => X"15691573156115421520153A156E156F156915741561157A15691572156F1568",
                    INIT_2F => X"1800191DB6740032B274603202F250007000FE03DDFF3E001D01700115001563",
                    INIT_30 => X"170320320064700067107001180C190102F22064180C1901C064D20117010057",
                    INIT_31 => X"22B5180C02A6191470001C00140015041780180019077001005A180819240064",
                    INIT_32 => X"677066605000D20292059000F710D600005A18081926230FD20100571800191D",
                    INIT_33 => X"6660171016007001004812180048026000480270B65464606570005A18081928",
                    INIT_34 => X"6660655064407001B6746342B7009601D20292046343D2039205D20870006770",
                    INIT_35 => X"12600048120000481200B634206417401901012E180C19282750064070006770",
                    INIT_36 => X"0048120000481201B6340036035A13002399B674004852020230004812000048",
                    INIT_37 => X"127F004812000048120000481203B6340036B674004812200048120C00481200",
                    INIT_38 => X"0048122A004812410048121F0048120900481202B6340036B674004812FF0048",
                    INIT_39 => X"0430B6345000B6740036B6F40036039E131D6399F30ED280B31DB2000036B674",
                    INIT_3A => X"02700047028000470290004712035000B67463A1C3401301E2300048A2301409",
                    INIT_3B => X"130123C1D3FFE230004713000047027000470280004702900047120320440047",
                    INIT_3C => X"03E1004402D00047027000470280004702900047120203E40D205000B67423BB",
                    INIT_3D => X"63DDD20103E1004402700047028000470290004712D803E4500003E663CFD201",
                    INIT_3E => X"0000000000000000000000000000000020441204204412062044004712055000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"88888888888AA822228A222228C2AAB5480808099DDDD8DDAAAC00B56228A6AA",
                   INITP_01 => X"D3760DDDDDDDD37760DDE82AA8274D02A08A020282222D360B58A88888888888",
                   INITP_02 => X"7820581D77755680888A00D7575D78255580A0228018222222220882DDDDDDDD",
                   INITP_03 => X"80D80D8DE800D380202D4D54E80235572D74D8037815580D35421DDB5D5557F7",
                   INITP_04 => X"3B76AD0D388B0CD83745588C0015E0DDFAEC0E0A0ACD756D0BD57B7780EE080D",
                   INITP_05 => X"0ABAC57AAAAAAAAAAAAAAAAAA574D8B58D8822F5B755BB5A2222DBD6128A0A88",
                   INITP_06 => X"2222A88888A8A8222228205D57B58C2D438889605C35836088C003822B70A0D2",
                   INITP_07 => X"00000000000088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA2",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"D920004408200047092000470047129FB001B03171FF003AB6740036B174003A",
                    INIT_01 => X"201E00D70368009E03B203D4601BD040900017001800190720132014F215F820",
                    INIT_02 => X"500095F690001507E0309511900095E9900015B9500035DFD000D57B9000D561",
                    INIT_03 => X"90010000203E108011961298203E1040110D1203203E1010112712005000150A",
                    INIT_04 => X"1200B654500092046049D2019205D208B6645000B67400475000603EB200B100",
                    INIT_05 => X"0620006607200048004E38FB2066004E38FB5000027000480280004802900048",
                    INIT_06 => X"004812040048120000481200B6545000B6740048004E5804191F0064191E5000",
                    INIT_07 => X"004812FF004812FF004812FF0048120100481201004812A8004812C000481280",
                    INIT_08 => X"004812C0004812030048120200481201004812DC004812080048120000481200",
                    INIT_09 => X"1800193950006098C67016010048A2602111B6740048120200481201004812A8",
                    INIT_0A => X"009817131600004812040048120000481200B6546069D280B2005000D2040057",
                    INIT_0B => X"B6740098171C1616004E190CB654012EB615B71419040064B71318081900B674",
                    INIT_0C => X"B2742064003217101901500061101000D1FFB11C0064D701B7261800191D0111",
                    INIT_0D => X"D2140057182819031000D1FF7000611070012064003217081901003AB6740036",
                    INIT_0E => X"20CBD21D20CBD21B20CBD21A20CBD21820D3D21C182C2143D2172133D20020F4",
                    INIT_0F => X"20CFD21C180C2320D21722F9D2000057180819031000D1002138D21320CBD21E",
                    INIT_10 => X"22F2D2152307D21320CBD21020CBD21E20CBD21D20CBD21B20CBD21A20CBD218",
                    INIT_11 => X"18C8006118A80061188800611868006118481700006118280061180817085000",
                    INIT_12 => X"004E5804165017001904710000641701190000CB182800CB1808006118E80061",
                    INIT_13 => X"7D007C007B007A0000641702182C1901203200641701182C1901206602600048",
                    INIT_14 => X"F700D60073006151FFFFDC0470006FF06CC07001005A1828192650007F007E00",
                    INIT_15 => X"004812380048026000480270B654005A182819280B700A601000F700D6002184",
                    INIT_16 => X"C4A02172D3FF2172D20A216DFFFFDC04350014011301E2300048140015001300",
                    INIT_17 => X"DFFF7001006417401901012E182C192827900680B6740950084063306163E5B0",
                    INIT_18 => X"FB00DA0070006FF06CC06BB06AA06990688067706330700150007000DFFF4192",
                    INIT_19 => X"1FFE61A6F40AF50DD302B401B500500013001F014199DF0001DBDF006BA01000",
                    INIT_1A => X"DD8061C002C41ADD1B0269906880677061B060A050001A351B021C005000DD80",
                    INIT_1B => X"1D0021A2E1B51401170161C0C250A54021BFD20003A9190718001740141561BF",
                    INIT_1C => X"900000260020A5A01B0D1A07170050001E0070016990688067706B106A007000",
                    INIT_1D => X"0D2003A9170018001907500061CCCAB01A01475061D190014900480047061004",
                    INIT_1E => X"A1A201C950001A3F1B0217001800190661EED220B205620602C41ACF1B021C00",
                    INIT_1F => X"50001A3F1B021C056200D27350001A3B1B021C0361FAD26521EB1C0261F4D262",
                    INIT_20 => X"50001A101B021C04A1A201C961A202C41AD51B0250001A311B021C0361EBD272",
                    INIT_21 => X"A4E01E005000223F7000FE1FDDFF3E001D0170010032621ED300700063307001",
                    INIT_22 => X"1E001D0003E6221F221FDE0F222DCE301E01700039003800170103C370016240",
                    INIT_23 => X"7001700003E603D47001223F7000170018FE190670014BA01A321B02023F5000",
                    INIT_24 => X"151F1C00170018FF19066D706B806A907001005A18281924225FDC0070006CC0",
                    INIT_25 => X"1924226CDC03226CDC041D006CC067D068B069A002B5182C02A6193470001440",
                    INIT_26 => X"0290700120D3182C02B5182C02A670001440151F70011934226EDC05005A1828",
                    INIT_27 => X"18007001035A700063806280D902227F00CB00D3627FD100180C01806279D901",
                    INIT_28 => X"A28079341800F20B00571903F20A005718081900F109F800039E1300F8005880",
                    INIT_29 => X"0280004802900048026000480270B65470006820226A7000628FD80C18010296",
                    INIT_2A => X"029622B3D20022AFDC0239003800170103A970015000700137001601B6740048",
                    INIT_2B => X"005A19205000D21700571903006417201901012E19245000700062A7B5009401",
                    INIT_2C => X"154722C53B001A0114015000C560A6401000D5004BA01400500062BAF720D600",
                    INIT_2D => X"1574157515411500155F152F152015541553154F15501500152F152015541545",
                    INIT_2E => X"15691573156115421520153A156E156F156915741561157A15691572156F1568",
                    INIT_2F => X"1800191DB6740032B274603202F250007000FE03DDFF3E001D01700115001563",
                    INIT_30 => X"170320320064700067107001180C190102F22064180C1901C064D20117010057",
                    INIT_31 => X"22B5180C02A6191470001C00140015041780180019077001005A180819240064",
                    INIT_32 => X"677066605000D20292059000F710D600005A18081926230FD20100571800191D",
                    INIT_33 => X"6660171016007001004812180048026000480270B65464606570005A18081928",
                    INIT_34 => X"6660655064407001B6746342B7009601D20292046343D2039205D20870006770",
                    INIT_35 => X"12600048120000481200B634206417401901012E180C19282750064070006770",
                    INIT_36 => X"0048120000481201B6340036035A13002399B674004852020230004812000048",
                    INIT_37 => X"127F004812000048120000481203B6340036B674004812200048120C00481200",
                    INIT_38 => X"0048122A004812410048121F0048120900481202B6340036B674004812FF0048",
                    INIT_39 => X"0430B6345000B6740036B6F40036039E131D6399F30ED280B31DB2000036B674",
                    INIT_3A => X"02700047028000470290004712035000B67463A1C3401301E2300048A2301409",
                    INIT_3B => X"130123C1D3FFE230004713000047027000470280004702900047120320440047",
                    INIT_3C => X"03E1004402D00047027000470280004702900047120203E40D205000B67423BB",
                    INIT_3D => X"63DDD20103E1004402700047028000470290004712D803E4500003E663CFD201",
                    INIT_3E => X"0000000000000000000000000000000020441204204412062044004712055000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"88888888888AA822228A222228C2AAB5480808099DDDD8DDAAAC00B56228A6AA",
                   INITP_01 => X"D3760DDDDDDDD37760DDE82AA8274D02A08A020282222D360B58A88888888888",
                   INITP_02 => X"7820581D77755680888A00D7575D78255580A0228018222222220882DDDDDDDD",
                   INITP_03 => X"80D80D8DE800D380202D4D54E80235572D74D8037815580D35421DDB5D5557F7",
                   INITP_04 => X"3B76AD0D388B0CD83745588C0015E0DDFAEC0E0A0ACD756D0BD57B7780EE080D",
                   INITP_05 => X"0ABAC57AAAAAAAAAAAAAAAAAA574D8B58D8822F5B755BB5A2222DBD6128A0A88",
                   INITP_06 => X"2222A88888A8A8222228205D57B58C2D438889605C35836088C003822B70A0D2",
                   INITP_07 => X"00000000000088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA2",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_2k_generate;
  --
  --	
  ram_4k_generate : if (C_RAM_SIZE_KWORDS = 4) generate
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      data_in_a <= "000000000000000000000000000000000000";
      --
      s6_a11_flop: FD
      port map (  D => address(11),
                  Q => pipe_a11,
                  C => clk);
      --
      s6_4k_mux0_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(0),
                I1 => data_out_a_hl(0),
                I2 => data_out_a_ll(1),
                I3 => data_out_a_hl(1),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(0),
                O6 => instruction(1));
      --
      s6_4k_mux2_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(2),
                I1 => data_out_a_hl(2),
                I2 => data_out_a_ll(3),
                I3 => data_out_a_hl(3),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(2),
                O6 => instruction(3));
      --
      s6_4k_mux4_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(4),
                I1 => data_out_a_hl(4),
                I2 => data_out_a_ll(5),
                I3 => data_out_a_hl(5),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(4),
                O6 => instruction(5));
      --
      s6_4k_mux6_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(6),
                I1 => data_out_a_hl(6),
                I2 => data_out_a_ll(7),
                I3 => data_out_a_hl(7),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(6),
                O6 => instruction(7));
      --
      s6_4k_mux8_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(32),
                I1 => data_out_a_hl(32),
                I2 => data_out_a_lh(0),
                I3 => data_out_a_hh(0),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(8),
                O6 => instruction(9));
      --
      s6_4k_mux10_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(1),
                I1 => data_out_a_hh(1),
                I2 => data_out_a_lh(2),
                I3 => data_out_a_hh(2),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(10),
                O6 => instruction(11));
      --
      s6_4k_mux12_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(3),
                I1 => data_out_a_hh(3),
                I2 => data_out_a_lh(4),
                I3 => data_out_a_hh(4),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(12),
                O6 => instruction(13));
      --
      s6_4k_mux14_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(5),
                I1 => data_out_a_hh(5),
                I2 => data_out_a_lh(6),
                I3 => data_out_a_hh(6),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(14),
                O6 => instruction(15));
      --
      s6_4k_mux16_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(7),
                I1 => data_out_a_hh(7),
                I2 => data_out_a_lh(32),
                I3 => data_out_a_hh(32),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(16),
                O6 => instruction(17));
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_ll <= "000" & data_out_b_ll(32) & "000000000000000000000000" & data_out_b_ll(7 downto 0);
        data_in_b_lh <= "000" & data_out_b_lh(32) & "000000000000000000000000" & data_out_b_lh(7 downto 0);
        data_in_b_hl <= "000" & data_out_b_hl(32) & "000000000000000000000000" & data_out_b_hl(7 downto 0);
        data_in_b_hh <= "000" & data_out_b_hh(32) & "000000000000000000000000" & data_out_b_hh(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b_l(3 downto 0) <= "0000";
        we_b_h(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
        jtag_dout <= data_out_b_lh(32) & data_out_b_lh(7 downto 0) & data_out_b_ll(32) & data_out_b_ll(7 downto 0);
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_lh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_ll <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        data_in_b_hh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_hl <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        --
        s6_4k_jtag_we_lut: LUT6_2
        generic map (INIT => X"8000000020000000")
        port map( I0 => jtag_we,
                  I1 => jtag_addr(11),
                  I2 => '1',
                  I3 => '1',
                  I4 => '1',
                  I5 => '1',
                  O5 => jtag_we_l,
                  O6 => jtag_we_h);
        --
        we_b_l(3 downto 0) <= jtag_we_l & jtag_we_l & jtag_we_l & jtag_we_l;
        we_b_h(3 downto 0) <= jtag_we_h & jtag_we_h & jtag_we_h & jtag_we_h;
        --
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
        --
        s6_4k_jtag_mux0_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(0),
                  I1 => data_out_b_hl(0),
                  I2 => data_out_b_ll(1),
                  I3 => data_out_b_hl(1),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(0),
                  O6 => jtag_dout(1));
        --
        s6_4k_jtag_mux2_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(2),
                  I1 => data_out_b_hl(2),
                  I2 => data_out_b_ll(3),
                  I3 => data_out_b_hl(3),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(2),
                  O6 => jtag_dout(3));
        --
        s6_4k_jtag_mux4_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(4),
                  I1 => data_out_b_hl(4),
                  I2 => data_out_b_ll(5),
                  I3 => data_out_b_hl(5),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(4),
                  O6 => jtag_dout(5));
        --
        s6_4k_jtag_mux6_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(6),
                  I1 => data_out_b_hl(6),
                  I2 => data_out_b_ll(7),
                  I3 => data_out_b_hl(7),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(6),
                  O6 => jtag_dout(7));
        --
        s6_4k_jtag_mux8_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(32),
                  I1 => data_out_b_hl(32),
                  I2 => data_out_b_lh(0),
                  I3 => data_out_b_hh(0),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(8),
                  O6 => jtag_dout(9));
        --
        s6_4k_jtag_mux10_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(1),
                  I1 => data_out_b_hh(1),
                  I2 => data_out_b_lh(2),
                  I3 => data_out_b_hh(2),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(10),
                  O6 => jtag_dout(11));
        --
        s6_4k_jtag_mux12_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(3),
                  I1 => data_out_b_hh(3),
                  I2 => data_out_b_lh(4),
                  I3 => data_out_b_hh(4),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(12),
                  O6 => jtag_dout(13));
        --
        s6_4k_jtag_mux14_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(5),
                  I1 => data_out_b_hh(5),
                  I2 => data_out_b_lh(6),
                  I3 => data_out_b_hh(6),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(14),
                  O6 => jtag_dout(15));
        --
        s6_4k_jtag_mux16_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(7),
                  I1 => data_out_b_hh(7),
                  I2 => data_out_b_lh(32),
                  I3 => data_out_b_hh(32),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(16),
                  O6 => jtag_dout(17));
      --
      end generate loader;
      --
      kcpsm6_rom_ll: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"1ED7689EB2D41B400000000713141520204420472047479F0131FF3A7436743A",
                    INIT_01 => X"01003E8096983E400D033E102700000A00F60007301100E900B900DF007B0061",
                    INIT_02 => X"206620484EFB664EFB00704880489048005400044901050864007447003E0000",
                    INIT_03 => X"48FF48FF48FF4801480148A848C04880480448004800540074484E041F641E00",
                    INIT_04 => X"003900987001486011744802480148A848C048034802480148DC480848004800",
                    INIT_05 => X"74981C164E0C542E151404641308007498130048044800480054698000000457",
                    INIT_06 => X"1457280300FF001001643208013A74367464321001001000FF1C640126001D11",
                    INIT_07 => X"CF1C0C2017F90057080300003813CB1ECB1DCB1BCB1ACB18D31C2C43173300F4",
                    INIT_08 => X"C861A861886168614800612861080800F2150713CB10CB1ECB1DCB1BCB1ACB18",
                    INIT_09 => X"0000000064022C013264012C016660484E0450000400640100CB28CB0861E861",
                    INIT_0A => X"483848604870545A282870600000008400000051FF0400F0C0015A2826000000",
                    INIT_0B => X"FF016440012E2C2890807450403063B0A072FF720A6DFF040001013048000000",
                    INIT_0C => X"FEA60A0D0201000000019900DB00A000000000F0C0B0A090807030010000FF92",
                    INIT_0D => X"00A2B50101C05040BF00A907004015BF80C0C4DD02908070B0A0003502000080",
                    INIT_0E => X"20A900000700CCB00150D10100000604002620A00D0700000001908070100000",
                    INIT_0F => X"003F02050073003B0203FA65EB02F462A2C9003F02000006EE200506C4CF0200",
                    INIT_10 => X"E000003F001FFF000101321E0000300100100204A2C9A2C4D50200310203EB72",
                    INIT_11 => X"0100E6D4013F0000FE0601A032023F000000E61F1F0F2D300100000001C30140",
                    INIT_12 => X"246C036C0400C0D0B0A0B52CA63400401F0000FF06708090015A28245F0000C0",
                    INIT_13 => X"00015A008080027FCBD37F000C8079019001D32CB52CA600401F01346E055A28",
                    INIT_14 => X"804890486048705400206A008F0C01968034000B57030A57080009009E000080",
                    INIT_15 => X"5A20001757036420012E240000A7000196B300AF02000001A901000100017448",
                    INIT_16 => X"747541005F2F2054534F50002F20544547C50001010060400000A00000BA2000",
                    INIT_17 => X"001D74327432F2000003FF000101006369736142203A6E6F6974617A69726F68",
                    INIT_18 => X"B50CA61400000004800007015A0824640332640010010C01F2640C0164010157",
                    INIT_19 => X"601000014818486048705460705A082870600002050010005A08260F0157001D",
                    INIT_1A => X"6048004800346440012E0C285040007060504001744200010204430305080070",
                    INIT_1B => X"7F4800480048033436744820480C48004800480134365A009974480230480048",
                    INIT_1C => X"3034007436F4369E1D990E801D003674482A4841481F4809480234367448FF48",
                    INIT_1D => X"01C1FF30470047704780479047034447704780479047030074A1400130483009",
                    INIT_1E => X"DD01E144704780479047D8E400E6CF01E144D04770478047904702E4200074BB",
                    INIT_1F => X"0000000000000000000000000000000000000000000000004404440644470500",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"1058001415181ADB256A4000408000000000000A20000001088955552C508822",
                   INITP_01 => X"20AACD82EA6A1A2CEDB5CD89DAFE95639D9776A30065B90AA5281D8000422000",
                   INITP_02 => X"402BFFFFFFFFA26245E205880000446C2A1500508544AE103154602C028A2E4A",
                   INITP_03 => X"00000000A0168019E400007001E800000000038001D946214009823111A28912",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_ll(31 downto 0),
                  DOPA => data_out_a_ll(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_ll(31 downto 0),
                  DOPB => data_out_b_ll(35 downto 32), 
                   DIB => data_in_b_ll(31 downto 0),
                  DIPB => data_in_b_ll(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_lh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"100001000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"C800100808091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA",
                    INIT_02 => X"03000300001C10001C28010001000100095B2849B06949695B285B0028B0D9D8",
                    INIT_03 => X"000900090009000900090009000900090009000900095B285B00002C0C000C28",
                    INIT_04 => X"0C0C28B0E38B0051105B00090009000900090009000900090009000900090009",
                    INIT_05 => X"5B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E900",
                    INIT_06 => X"E9000C0C88E8B8B0B810000B0C005B005910000B0C28B088E85880EB5B0C0C00",
                    INIT_07 => X"90E90C91E991E9000C0C88E890E990E990E990E990E990E990E90C90E990E990",
                    INIT_08 => X"0C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E990E9",
                    INIT_09 => X"BEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C00",
                    INIT_0A => X"0009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BFBF",
                    INIT_0B => X"EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A09",
                    INIT_0C => X"0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EFA0",
                    INIT_0D => X"0E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8EE",
                    INIT_0E => X"06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5B8",
                    INIT_0F => X"280D0D0EB1E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D0E",
                    INIT_10 => X"520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0E9",
                    INIT_11 => X"B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8B1",
                    INIT_12 => X"0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8B6",
                    INIT_13 => X"0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE000C",
                    INIT_14 => X"010001000100015BB8B411B8B1EC8C0151BC0C79000C79000C0C787C01097C2C",
                    INIT_15 => X"000CA8E9000C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B00",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB",
                    INIT_17 => X"0C0C5B0059B00128B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"110C010CB80E0A0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B00",
                    INIT_19 => X"B30B0BB80009000100015BB2B2000C0CB3B3A86949C8FBEB000C0C91E9000C0C",
                    INIT_1A => X"09000900095B100B0C000C0C9383B8B3B3B2B2B85BB1DBCB6949B1694969B8B3",
                    INIT_1B => X"090009000900095B005B000900090009000900095B000109115B002901000900",
                    INIT_1C => X"025B285B005B000109B1F9E95959005B000900090009000900095B005B000900",
                    INIT_1D => X"8991E97100090001000100010009100001000100010009285BB1E1897100518A",
                    INIT_1E => X"B169010001000100010009012801B16901000100010001000100090106285B11",
                    INIT_1F => X"0000000000000000000000000000000000000000000000001009100910000928",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"952AAA954AE7E521CB11956532EAAAAAAAAAABE55B5569FC2222AAAAFE0C56DF",
                   INITP_01 => X"8A2AE0984620E14164A16022412B201D64225418AB09126408C582555529AAAA",
                   INITP_02 => X"3F87FFFFFFFFC4ACAA5CD0F355B91B3A75E26B2A502A00CAFE333A4638758F22",
                   INITP_03 => X"000000ADB55ED5575AAB55CC7F43AABD55EAAEE556421CA61AA42494A81974C9",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_lh(31 downto 0),
                  DOPA => data_out_a_lh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_lh(31 downto 0),
                  DOPB => data_out_b_lh(35 downto 32), 
                   DIB => data_in_b_lh(31 downto 0),
                  DIPB => data_in_b_lh(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      --
      kcpsm6_rom_hl: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hl(31 downto 0),
                  DOPA => data_out_a_hl(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hl(31 downto 0),
                  DOPB => data_out_b_hl(35 downto 32), 
                   DIB => data_in_b_hl(31 downto 0),
                  DIPB => data_in_b_hl(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_hh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hh(31 downto 0),
                  DOPA => data_out_a_hh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hh(31 downto 0),
                  DOPB => data_out_b_hh(35 downto 32), 
                   DIB => data_in_b_hh(31 downto 0),
                  DIPB => data_in_b_hh(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"1ED7689EB2D41B400000000713141520204420472047479F0131FF3A7436743A",
                    INIT_01 => X"01003E8096983E400D033E102700000A00F60007301100E900B900DF007B0061",
                    INIT_02 => X"206620484EFB664EFB00704880489048005400044901050864007447003E0000",
                    INIT_03 => X"48FF48FF48FF4801480148A848C04880480448004800540074484E041F641E00",
                    INIT_04 => X"003900987001486011744802480148A848C048034802480148DC480848004800",
                    INIT_05 => X"74981C164E0C542E151404641308007498130048044800480054698000000457",
                    INIT_06 => X"1457280300FF001001643208013A74367464321001001000FF1C640126001D11",
                    INIT_07 => X"CF1C0C2017F90057080300003813CB1ECB1DCB1BCB1ACB18D31C2C43173300F4",
                    INIT_08 => X"C861A861886168614800612861080800F2150713CB10CB1ECB1DCB1BCB1ACB18",
                    INIT_09 => X"0000000064022C013264012C016660484E0450000400640100CB28CB0861E861",
                    INIT_0A => X"483848604870545A282870600000008400000051FF0400F0C0015A2826000000",
                    INIT_0B => X"FF016440012E2C2890807450403063B0A072FF720A6DFF040001013048000000",
                    INIT_0C => X"FEA60A0D0201000000019900DB00A000000000F0C0B0A090807030010000FF92",
                    INIT_0D => X"00A2B50101C05040BF00A907004015BF80C0C4DD02908070B0A0003502000080",
                    INIT_0E => X"20A900000700CCB00150D10100000604002620A00D0700000001908070100000",
                    INIT_0F => X"003F02050073003B0203FA65EB02F462A2C9003F02000006EE200506C4CF0200",
                    INIT_10 => X"E000003F001FFF000101321E0000300100100204A2C9A2C4D50200310203EB72",
                    INIT_11 => X"0100E6D4013F0000FE0601A032023F000000E61F1F0F2D300100000001C30140",
                    INIT_12 => X"246C036C0400C0D0B0A0B52CA63400401F0000FF06708090015A28245F0000C0",
                    INIT_13 => X"00015A008080027FCBD37F000C8079019001D32CB52CA600401F01346E055A28",
                    INIT_14 => X"804890486048705400206A008F0C01968034000B57030A57080009009E000080",
                    INIT_15 => X"5A20001757036420012E240000A7000196B300AF02000001A901000100017448",
                    INIT_16 => X"747541005F2F2054534F50002F20544547C50001010060400000A00000BA2000",
                    INIT_17 => X"001D74327432F2000003FF000101006369736142203A6E6F6974617A69726F68",
                    INIT_18 => X"B50CA61400000004800007015A0824640332640010010C01F2640C0164010157",
                    INIT_19 => X"601000014818486048705460705A082870600002050010005A08260F0157001D",
                    INIT_1A => X"6048004800346440012E0C285040007060504001744200010204430305080070",
                    INIT_1B => X"7F4800480048033436744820480C48004800480134365A009974480230480048",
                    INIT_1C => X"3034007436F4369E1D990E801D003674482A4841481F4809480234367448FF48",
                    INIT_1D => X"01C1FF30470047704780479047034447704780479047030074A1400130483009",
                    INIT_1E => X"DD01E144704780479047D8E400E6CF01E144D04770478047904702E4200074BB",
                    INIT_1F => X"0000000000000000000000000000000000000000000000004404440644470500",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"1058001415181ADB256A4000408000000000000A20000001088955552C508822",
                   INITP_01 => X"20AACD82EA6A1A2CEDB5CD89DAFE95639D9776A30065B90AA5281D8000422000",
                   INITP_02 => X"402BFFFFFFFFA26245E205880000446C2A1500508544AE103154602C028A2E4A",
                   INITP_03 => X"00000000A0168019E400007001E800000000038001D946214009823111A28912",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"100001000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"C800100808091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA",
                    INIT_02 => X"03000300001C10001C28010001000100095B2849B06949695B285B0028B0D9D8",
                    INIT_03 => X"000900090009000900090009000900090009000900095B285B00002C0C000C28",
                    INIT_04 => X"0C0C28B0E38B0051105B00090009000900090009000900090009000900090009",
                    INIT_05 => X"5B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E900",
                    INIT_06 => X"E9000C0C88E8B8B0B810000B0C005B005910000B0C28B088E85880EB5B0C0C00",
                    INIT_07 => X"90E90C91E991E9000C0C88E890E990E990E990E990E990E990E90C90E990E990",
                    INIT_08 => X"0C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E990E9",
                    INIT_09 => X"BEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C00",
                    INIT_0A => X"0009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BFBF",
                    INIT_0B => X"EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A09",
                    INIT_0C => X"0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EFA0",
                    INIT_0D => X"0E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8EE",
                    INIT_0E => X"06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5B8",
                    INIT_0F => X"280D0D0EB1E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D0E",
                    INIT_10 => X"520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0E9",
                    INIT_11 => X"B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8B1",
                    INIT_12 => X"0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8B6",
                    INIT_13 => X"0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE000C",
                    INIT_14 => X"010001000100015BB8B411B8B1EC8C0151BC0C79000C79000C0C787C01097C2C",
                    INIT_15 => X"000CA8E9000C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B00",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB",
                    INIT_17 => X"0C0C5B0059B00128B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"110C010CB80E0A0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B00",
                    INIT_19 => X"B30B0BB80009000100015BB2B2000C0CB3B3A86949C8FBEB000C0C91E9000C0C",
                    INIT_1A => X"09000900095B100B0C000C0C9383B8B3B3B2B2B85BB1DBCB6949B1694969B8B3",
                    INIT_1B => X"090009000900095B005B000900090009000900095B000109115B002901000900",
                    INIT_1C => X"025B285B005B000109B1F9E95959005B000900090009000900095B005B000900",
                    INIT_1D => X"8991E97100090001000100010009100001000100010009285BB1E1897100518A",
                    INIT_1E => X"B169010001000100010009012801B16901000100010001000100090106285B11",
                    INIT_1F => X"0000000000000000000000000000000000000000000000001009100910000928",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"952AAA954AE7E521CB11956532EAAAAAAAAAABE55B5569FC2222AAAAFE0C56DF",
                   INITP_01 => X"8A2AE0984620E14164A16022412B201D64225418AB09126408C582555529AAAA",
                   INITP_02 => X"3F87FFFFFFFFC4ACAA5CD0F355B91B3A75E26B2A502A00CAFE333A4638758F22",
                   INITP_03 => X"000000ADB55ED5575AAB55CC7F43AABD55EAAEE556421CA61AA42494A81974C9",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"1ED7689EB2D41B400000000713141520204420472047479F0131FF3A7436743A",
                    INIT_01 => X"01003E8096983E400D033E102700000A00F60007301100E900B900DF007B0061",
                    INIT_02 => X"206620484EFB664EFB00704880489048005400044901050864007447003E0000",
                    INIT_03 => X"48FF48FF48FF4801480148A848C04880480448004800540074484E041F641E00",
                    INIT_04 => X"003900987001486011744802480148A848C048034802480148DC480848004800",
                    INIT_05 => X"74981C164E0C542E151404641308007498130048044800480054698000000457",
                    INIT_06 => X"1457280300FF001001643208013A74367464321001001000FF1C640126001D11",
                    INIT_07 => X"CF1C0C2017F90057080300003813CB1ECB1DCB1BCB1ACB18D31C2C43173300F4",
                    INIT_08 => X"C861A861886168614800612861080800F2150713CB10CB1ECB1DCB1BCB1ACB18",
                    INIT_09 => X"0000000064022C013264012C016660484E0450000400640100CB28CB0861E861",
                    INIT_0A => X"483848604870545A282870600000008400000051FF0400F0C0015A2826000000",
                    INIT_0B => X"FF016440012E2C2890807450403063B0A072FF720A6DFF040001013048000000",
                    INIT_0C => X"FEA60A0D0201000000019900DB00A000000000F0C0B0A090807030010000FF92",
                    INIT_0D => X"00A2B50101C05040BF00A907004015BF80C0C4DD02908070B0A0003502000080",
                    INIT_0E => X"20A900000700CCB00150D10100000604002620A00D0700000001908070100000",
                    INIT_0F => X"003F02050073003B0203FA65EB02F462A2C9003F02000006EE200506C4CF0200",
                    INIT_10 => X"E000003F001FFF000101321E0000300100100204A2C9A2C4D50200310203EB72",
                    INIT_11 => X"0100E6D4013F0000FE0601A032023F000000E61F1F0F2D300100000001C30140",
                    INIT_12 => X"246C036C0400C0D0B0A0B52CA63400401F0000FF06708090015A28245F0000C0",
                    INIT_13 => X"00015A008080027FCBD37F000C8079019001D32CB52CA600401F01346E055A28",
                    INIT_14 => X"804890486048705400206A008F0C01968034000B57030A57080009009E000080",
                    INIT_15 => X"5A20001757036420012E240000A7000196B300AF02000001A901000100017448",
                    INIT_16 => X"747541005F2F2054534F50002F20544547C50001010060400000A00000BA2000",
                    INIT_17 => X"001D74327432F2000003FF000101006369736142203A6E6F6974617A69726F68",
                    INIT_18 => X"B50CA61400000004800007015A0824640332640010010C01F2640C0164010157",
                    INIT_19 => X"601000014818486048705460705A082870600002050010005A08260F0157001D",
                    INIT_1A => X"6048004800346440012E0C285040007060504001744200010204430305080070",
                    INIT_1B => X"7F4800480048033436744820480C48004800480134365A009974480230480048",
                    INIT_1C => X"3034007436F4369E1D990E801D003674482A4841481F4809480234367448FF48",
                    INIT_1D => X"01C1FF30470047704780479047034447704780479047030074A1400130483009",
                    INIT_1E => X"DD01E144704780479047D8E400E6CF01E144D04770478047904702E4200074BB",
                    INIT_1F => X"0000000000000000000000000000000000000000000000004404440644470500",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"1058001415181ADB256A4000408000000000000A20000001088955552C508822",
                   INITP_01 => X"20AACD82EA6A1A2CEDB5CD89DAFE95639D9776A30065B90AA5281D8000422000",
                   INITP_02 => X"402BFFFFFFFFA26245E205880000446C2A1500508544AE103154602C028A2E4A",
                   INITP_03 => X"00000000A0168019E400007001E800000000038001D946214009823111A28912",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"100001000101B068480B0C0C1090F9FCEC000400040000095858B8005B005800",
                    INIT_01 => X"C800100808091008080910080809288A28CAC88AF0CAC8CAC88A281AE8EAC8EA",
                    INIT_02 => X"03000300001C10001C28010001000100095B2849B06949695B285B0028B0D9D8",
                    INIT_03 => X"000900090009000900090009000900090009000900095B285B00002C0C000C28",
                    INIT_04 => X"0C0C28B0E38B0051105B00090009000900090009000900090009000900090009",
                    INIT_05 => X"5B000B0B000C5B005B5B0C005B0C0C5B000B0B0009000900095BB0E959A8E900",
                    INIT_06 => X"E9000C0C88E8B8B0B810000B0C005B005910000B0C28B088E85880EB5B0C0C00",
                    INIT_07 => X"90E90C91E991E9000C0C88E890E990E990E990E990E990E990E90C90E990E990",
                    INIT_08 => X"0C000C000C000C000C0B000C000C0B2891E991E990E990E990E990E990E990E9",
                    INIT_09 => X"BEBEBDBD000B0C0C10000B0C0C100100002C0B0B0CB8000B0C000C000C000C00",
                    INIT_0A => X"0009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8B7B6B8000C0C28BFBF",
                    INIT_0B => X"EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FFEE9A8A8971000A0A09",
                    INIT_0C => X"0FB0FAFAE95A5A28098FA0EF80EF3588FDEDB8B7B6B5B5B4B4B3B1B8A8B8EFA0",
                    INIT_0D => X"0E10F08A8BB0E15290E9010C0C0B0AB0EEB0010D0DB4B4B3B0B0280D0D0EA8EE",
                    INIT_0E => X"06010B0C0C28B0E58D23B0C8A4A4A308C80000520D0D0B280FB8B4B4B3B5B5B8",
                    INIT_0F => X"280D0D0EB1E9280D0D0EB0E9100EB0E9D000280D0D0B0C0CB0E959B1010D0D0E",
                    INIT_10 => X"520F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0ED000B0010D0D280D0D0EB0E9",
                    INIT_11 => X"B8B80101B811B80B0C0CB8250D0D01280F0E0111916F91E78FB89C9C8B01B8B1",
                    INIT_12 => X"0C91EE91EE0EB6B3B4B4010C010CB80A0A0E0B0C0CB6B5B5B8000C0C91EEB8B6",
                    INIT_13 => X"0CB801B8B1B1EC110000B1E80C00B1EC01B8100C010C01B80A0AB80C91EE000C",
                    INIT_14 => X"010001000100015BB8B411B8B1EC8C0151BC0C79000C79000C0C787C01097C2C",
                    INIT_15 => X"000CA8E9000C000B0C000C28B8B1DACA0191E991EE9C9C8B01B828B89B8B5B00",
                    INIT_16 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388EA250A28B1FBEB",
                    INIT_17 => X"0C0C5B0059B00128B87FEE9F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_18 => X"110C010CB80E0A0A0B0C0CB8000C0C000B1000B8B3B80C0C01100C0CE0E90B00",
                    INIT_19 => X"B30B0BB80009000100015BB2B2000C0CB3B3A86949C8FBEB000C0C91E9000C0C",
                    INIT_1A => X"09000900095B100B0C000C0C9383B8B3B3B2B2B85BB1DBCB6949B1694969B8B3",
                    INIT_1B => X"090009000900095B005B000900090009000900095B000109115B002901000900",
                    INIT_1C => X"025B285B005B000109B1F9E95959005B000900090009000900095B005B000900",
                    INIT_1D => X"8991E97100090001000100010009100001000100010009285BB1E1897100518A",
                    INIT_1E => X"B169010001000100010009012801B16901000100010001000100090106285B11",
                    INIT_1F => X"0000000000000000000000000000000000000000000000001009100910000928",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"952AAA954AE7E521CB11956532EAAAAAAAAAABE55B5569FC2222AAAAFE0C56DF",
                   INITP_01 => X"8A2AE0984620E14164A16022412B201D64225418AB09126408C582555529AAAA",
                   INITP_02 => X"3F87FFFFFFFFC4ACAA5CD0F355B91B3A75E26B2A502A00CAFE333A4638758F22",
                   INITP_03 => X"000000ADB55ED5575AAB55CC7F43AABD55EAAEE556421CA61AA42494A81974C9",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_4k_generate;	              
  --
  --
  --
  --
  -- JTAG Loader
  --
  instantiate_loader : if (C_JTAG_LOADER_ENABLE = 1) generate
  --
    jtag_loader_6_inst : jtag_loader_6
    generic map(              C_FAMILY => C_FAMILY,
                       C_NUM_PICOBLAZE => 1,
                  C_JTAG_LOADER_ENABLE => C_JTAG_LOADER_ENABLE,
                 C_BRAM_MAX_ADDR_WIDTH => BRAM_ADDRESS_WIDTH,
	                  C_ADDR_WIDTH_0 => BRAM_ADDRESS_WIDTH)
    port map( picoblaze_reset => rdl_bus,
                      jtag_en => jtag_en,
                     jtag_din => jtag_din,
                    jtag_addr => jtag_addr(BRAM_ADDRESS_WIDTH-1 downto 0),
                     jtag_clk => jtag_clk,
                      jtag_we => jtag_we,
                  jtag_dout_0 => jtag_dout,
                  jtag_dout_1 => jtag_dout, -- ports 1-7 are not used
                  jtag_dout_2 => jtag_dout, -- in a 1 device debug 
                  jtag_dout_3 => jtag_dout, -- session.  However, Synplify
                  jtag_dout_4 => jtag_dout, -- etc require all ports to
                  jtag_dout_5 => jtag_dout, -- be connected
                  jtag_dout_6 => jtag_dout,
                  jtag_dout_7 => jtag_dout);
    --  
  end generate instantiate_loader;
  --
end low_level_definition;
--
--
-------------------------------------------------------------------------------------------
--
-- JTAG Loader 
--
-------------------------------------------------------------------------------------------
--
--
-- JTAG Loader 6 - Version 6.00
-- Kris Chaplin 4 February 2010
-- Ken Chapman 15 August 2011 - Revised coding style
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
library unisim;
use unisim.vcomponents.all;
--
entity jtag_loader_6 is
generic(              C_JTAG_LOADER_ENABLE : integer := 1;
                                  C_FAMILY : string := "V6";
                           C_NUM_PICOBLAZE : integer := 1;
                     C_BRAM_MAX_ADDR_WIDTH : integer := 10;
        C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                              C_JTAG_CHAIN : integer := 2;
                            C_ADDR_WIDTH_0 : integer := 10;
                            C_ADDR_WIDTH_1 : integer := 10;
                            C_ADDR_WIDTH_2 : integer := 10;
                            C_ADDR_WIDTH_3 : integer := 10;
                            C_ADDR_WIDTH_4 : integer := 10;
                            C_ADDR_WIDTH_5 : integer := 10;
                            C_ADDR_WIDTH_6 : integer := 10;
                            C_ADDR_WIDTH_7 : integer := 10);
port(   picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
               jtag_din : out std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
              jtag_addr : out std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0) := (others => '0');
               jtag_clk : out std_logic := '0';
                jtag_we : out std_logic := '0';
            jtag_dout_0 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_1 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_2 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_3 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_4 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_5 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_6 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_7 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end jtag_loader_6;
--
architecture Behavioral of jtag_loader_6 is
  --
  signal num_picoblaze       : std_logic_vector(2 downto 0);
  signal picoblaze_instruction_data_width : std_logic_vector(4 downto 0);
  --
  signal drck                : std_logic;
  signal shift_clk           : std_logic;
  signal shift_din           : std_logic;
  signal shift_dout          : std_logic;
  signal shift               : std_logic;
  signal capture             : std_logic;
  --
  signal control_reg_ce      : std_logic;
  signal bram_ce             : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal bus_zero            : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  signal jtag_en_int         : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal jtag_en_expanded    : std_logic_vector(7 downto 0) := (others => '0');
  signal jtag_addr_int       : std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
  signal jtag_din_int        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal control_din         : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout_int    : std_logic_vector(7 downto 0):= (others => '0');
  signal bram_dout_int       : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
  signal jtag_we_int         : std_logic;
  signal jtag_clk_int        : std_logic;
  signal bram_ce_valid       : std_logic;
  signal din_load            : std_logic;
  --
  signal jtag_dout_0_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_1_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_2_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_3_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_4_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_5_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_6_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_7_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal picoblaze_reset_int : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  --        
begin
  bus_zero <= (others => '0');
  --
  jtag_loader_gen: if (C_JTAG_LOADER_ENABLE = 1) generate
    --
    -- Insert BSCAN primitive for target device architecture.
    --
    BSCAN_SPARTAN6_gen: if (C_FAMILY="S6") generate
    begin
      BSCAN_BLOCK_inst : BSCAN_SPARTAN6
      generic map ( JTAG_CHAIN => C_JTAG_CHAIN)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_SPARTAN6_gen;   
    --
    BSCAN_VIRTEX6_gen: if (C_FAMILY="V6") generate
    begin
      BSCAN_BLOCK_inst: BSCAN_VIRTEX6
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => FALSE)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_VIRTEX6_gen;   
    --
    BSCAN_7SERIES_gen: if (C_FAMILY="7S") generate
    begin
      BSCAN_BLOCK_inst: BSCANE2
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => "FALSE")
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_7SERIES_gen;   
    --
    --
    -- Insert clock buffer to ensure reliable shift operations.
    --
    upload_clock: BUFG
    port map( I => drck,
              O => shift_clk);
    --        
    --        
    --  Shift Register      
    --        
    --
    control_reg_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk = '1' then
        if (shift = '1') then
          control_reg_ce <= shift_din;
        end if;
      end if;
    end process control_reg_ce_shift;
    --        
    bram_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          if(C_NUM_PICOBLAZE > 1) then
            for i in 0 to C_NUM_PICOBLAZE-2 loop
              bram_ce(i+1) <= bram_ce(i);
            end loop;
          end if;
          bram_ce(0) <= control_reg_ce;
        end if;
      end if;
    end process bram_ce_shift;
    --        
    bram_we_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          jtag_we_int <= bram_ce(C_NUM_PICOBLAZE-1);
        end if;
      end if;
    end process bram_we_shift;
    --        
    bram_a_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          for i in 0 to C_BRAM_MAX_ADDR_WIDTH-2 loop
            jtag_addr_int(i+1) <= jtag_addr_int(i);
          end loop;
          jtag_addr_int(0) <= jtag_we_int;
        end if;
      end if;
    end process bram_a_shift;
    --        
    bram_d_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (din_load = '1') then
          jtag_din_int <= bram_dout_int;
         elsif (shift = '1') then
          for i in 0 to C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-2 loop
            jtag_din_int(i+1) <= jtag_din_int(i);
          end loop;
          jtag_din_int(0) <= jtag_addr_int(C_BRAM_MAX_ADDR_WIDTH-1);
        end if;
      end if;
    end process bram_d_shift;
    --
    shift_dout <= jtag_din_int(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1);
    --
    --
    din_load_select:process (bram_ce, din_load, capture, bus_zero, control_reg_ce) 
    begin
      if ( bram_ce = bus_zero ) then
        din_load <= capture and control_reg_ce;
       else
        din_load <= capture;
      end if;
    end process din_load_select;
    --
    --
    -- Control Registers 
    --
    num_picoblaze <= conv_std_logic_vector(C_NUM_PICOBLAZE-1,3);
    picoblaze_instruction_data_width <= conv_std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1,5);
    --	
    control_registers: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '0') and (control_reg_ce = '1') then
          case (jtag_addr_int(3 downto 0)) is 
            when "0000" => -- 0 = version - returns (7 downto 4) illustrating number of PB
                           --               and (3 downto 0) picoblaze instruction data width
                           control_dout_int <= num_picoblaze & picoblaze_instruction_data_width;
            when "0001" => -- 1 = PicoBlaze 0 reset / status
                           if (C_NUM_PICOBLAZE >= 1) then 
                            control_dout_int <= picoblaze_reset_int(0) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_0-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0010" => -- 2 = PicoBlaze 1 reset / status
                           if (C_NUM_PICOBLAZE >= 2) then 
                             control_dout_int <= picoblaze_reset_int(1) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_1-1,5) );
                            else 
                             control_dout_int <= (others => '0');
                           end if;
            when "0011" => -- 3 = PicoBlaze 2 reset / status
                           if (C_NUM_PICOBLAZE >= 3) then 
                            control_dout_int <= picoblaze_reset_int(2) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_2-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0100" => -- 4 = PicoBlaze 3 reset / status
                           if (C_NUM_PICOBLAZE >= 4) then 
                            control_dout_int <= picoblaze_reset_int(3) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_3-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0101" => -- 5 = PicoBlaze 4 reset / status
                           if (C_NUM_PICOBLAZE >= 5) then 
                            control_dout_int <= picoblaze_reset_int(4) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_4-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0110" => -- 6 = PicoBlaze 5 reset / status
                           if (C_NUM_PICOBLAZE >= 6) then 
                            control_dout_int <= picoblaze_reset_int(5) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_5-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0111" => -- 7 = PicoBlaze 6 reset / status
                           if (C_NUM_PICOBLAZE >= 7) then 
                            control_dout_int <= picoblaze_reset_int(6) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_6-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1000" => -- 8 = PicoBlaze 7 reset / status
                           if (C_NUM_PICOBLAZE >= 8) then 
                            control_dout_int <= picoblaze_reset_int(7) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_7-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1111" => control_dout_int <= conv_std_logic_vector(C_BRAM_MAX_ADDR_WIDTH -1,8);
            when others => control_dout_int <= (others => '1');
          end case;
        else 
          control_dout_int <= (others => '0');
        end if;
      end if;
    end process control_registers;
    -- 
    control_dout(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-8) <= control_dout_int;
    --
    pb_reset: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '1') and (control_reg_ce = '1') then
          picoblaze_reset_int(C_NUM_PICOBLAZE-1 downto 0) <= control_din(C_NUM_PICOBLAZE-1 downto 0);
        end if;
      end if;
    end process pb_reset;    
    --
    --
    -- Assignments 
    --
    control_dout (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-9 downto 0) <= (others => '0') when (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH > 8);
    --
    -- Qualify the blockram CS signal with bscan select output
    jtag_en_int <= bram_ce when bram_ce_valid = '1' else (others => '0');
    --      
    jtag_en_expanded(C_NUM_PICOBLAZE-1 downto 0) <= jtag_en_int;
    jtag_en_expanded(7 downto C_NUM_PICOBLAZE) <= (others => '0') when (C_NUM_PICOBLAZE < 8);
    --        
    bram_dout_int <= control_dout or jtag_dout_0_masked or jtag_dout_1_masked or jtag_dout_2_masked or jtag_dout_3_masked or jtag_dout_4_masked or jtag_dout_5_masked or jtag_dout_6_masked or jtag_dout_7_masked;
    --
    control_din <= jtag_din_int;
    --        
    jtag_dout_0_masked <= jtag_dout_0 when jtag_en_expanded(0) = '1' else (others => '0');
    jtag_dout_1_masked <= jtag_dout_1 when jtag_en_expanded(1) = '1' else (others => '0');
    jtag_dout_2_masked <= jtag_dout_2 when jtag_en_expanded(2) = '1' else (others => '0');
    jtag_dout_3_masked <= jtag_dout_3 when jtag_en_expanded(3) = '1' else (others => '0');
    jtag_dout_4_masked <= jtag_dout_4 when jtag_en_expanded(4) = '1' else (others => '0');
    jtag_dout_5_masked <= jtag_dout_5 when jtag_en_expanded(5) = '1' else (others => '0');
    jtag_dout_6_masked <= jtag_dout_6 when jtag_en_expanded(6) = '1' else (others => '0');
    jtag_dout_7_masked <= jtag_dout_7 when jtag_en_expanded(7) = '1' else (others => '0');
    --
    jtag_en <= jtag_en_int;
    jtag_din <= jtag_din_int;
    jtag_addr <= jtag_addr_int;
    jtag_clk <= jtag_clk_int;
    jtag_we <= jtag_we_int;
    picoblaze_reset <= picoblaze_reset_int;
    --        
  end generate jtag_loader_gen;
--
end Behavioral;
--
--
------------------------------------------------------------------------------------
--
-- END OF FILE m25p16_spi_uart_bridge.vhd
--
------------------------------------------------------------------------------------
