--
-------------------------------------------------------------------------------------------
-- Copyright  2010-2013, Xilinx, Inc.
-- This file contains confidential and proprietary information of Xilinx, Inc. and is
-- protected under U.S. and international copyright and other intellectual property laws.
-------------------------------------------------------------------------------------------
--
-- Disclaimer:
-- This disclaimer is not a license and does not grant any rights to the materials
-- distributed herewith. Except as otherwise provided in a valid license issued to
-- you by Xilinx, and to the maximum extent permitted by applicable law: (1) THESE
-- MATERIALS ARE MADE AVAILABLE "AS IS" AND WITH ALL FAULTS, AND XILINX HEREBY
-- DISCLAIMS ALL WARRANTIES AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY,
-- INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-INFRINGEMENT,
-- OR FITNESS FOR ANY PARTICULAR PURPOSE; and (2) Xilinx shall not be liable
-- (whether in contract or tort, including negligence, or under any other theory
-- of liability) for any loss or damage of any kind or nature related to, arising
-- under or in connection with these materials, including for any direct, or any
-- indirect, special, incidental, or consequential loss or damage (including loss
-- of data, profits, goodwill, or any type of loss or damage suffered as a result
-- of any action brought by a third party) even if such damage or loss was
-- reasonably foreseeable or Xilinx had been advised of the possibility of the same.
--
-- CRITICAL APPLICATIONS
-- Xilinx products are not designed or intended to be fail-safe, or for use in any
-- application requiring fail-safe performance, such as life-support or safety
-- devices or systems, Class III medical devices, nuclear facilities, applications
-- related to the deployment of airbags, or any other applications that could lead
-- to death, personal injury, or severe property or environmental damage
-- (individually and collectively, "Critical Applications"). Customer assumes the
-- sole risk and liability of any use of Xilinx products in Critical Applications,
-- subject only to applicable laws and regulations governing limitations on product
-- liability.
--
-- THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS PART OF THIS FILE AT ALL TIMES.
--
-------------------------------------------------------------------------------------------
--
--
-- Definition of a program memory for KCPSM6 including generic parameters for the 
-- convenient selection of device family, program memory size and the ability to include 
-- the JTAG Loader hardware for rapid software development.
--
-- This file is primarily for use during code development and it is recommended that the 
-- appropriate simplified program memory definition be used in a final production design. 
--
--    Generic                  Values             Comments
--    Parameter                Supported
--  
--    C_FAMILY                 "S6"               Spartan-6 device
--                             "V6"               Virtex-6 device
--                             "7S"               7-Series device 
--                                                  (Artix-7, Kintex-7, Virtex-7 or Zynq)
--
--    C_RAM_SIZE_KWORDS        1, 2 or 4          Size of program memory in K-instructions
--
--    C_JTAG_LOADER_ENABLE     0 or 1             Set to '1' to include JTAG Loader
--
-- Notes
--
-- If your design contains MULTIPLE KCPSM6 instances then only one should have the 
-- JTAG Loader enabled at a time (i.e. make sure that C_JTAG_LOADER_ENABLE is only set to 
-- '1' on one instance of the program memory). Advanced users may be interested to know 
-- that it is possible to connect JTAG Loader to multiple memories and then to use the 
-- JTAG Loader utility to specify which memory contents are to be modified. However, 
-- this scheme does require some effort to set up and the additional connectivity of the 
-- multiple BRAMs can impact the placement, routing and performance of the complete 
-- design. Please contact the author at Xilinx for more detailed information. 
--
-- Regardless of the size of program memory specified by C_RAM_SIZE_KWORDS, the complete 
-- 12-bit address bus is connected to KCPSM6. This enables the generic to be modified 
-- without requiring changes to the fundamental hardware definition. However, when the 
-- program memory is 1K then only the lower 10-bits of the address are actually used and 
-- the valid address range is 000 to 3FF hex. Likewise, for a 2K program only the lower 
-- 11-bits of the address are actually used and the valid address range is 000 to 7FF hex.
--
-- Programs are stored in Block Memory (BRAM) and the number of BRAM used depends on the 
-- size of the program and the device family. 
--
-- In a Spartan-6 device a BRAM is capable of holding 1K instructions. Hence a 2K program 
-- will require 2 BRAMs to be used and a 4K program will require 4 BRAMs to be used. It 
-- should be noted that a 4K program is not such a natural fit in a Spartan-6 device and 
-- the implementation also requires a small amount of logic resulting in slightly lower 
-- performance. A Spartan-6 BRAM can also be split into two 9k-bit memories suggesting 
-- that a program containing up to 512 instructions could be implemented. However, there 
-- is a silicon errata which makes this unsuitable and therefore it is not supported by 
-- this file.
--
-- In a Virtex-6 or any 7-Series device a BRAM is capable of holding 2K instructions so 
-- obviously a 2K program requires only a single BRAM. Each BRAM can also be divided into 
-- 2 smaller memories supporting programs of 1K in half of a 36k-bit BRAM (generally 
-- reported as being an 18k-bit BRAM). For a program of 4K instructions, 2 BRAMs are used.
--
--
-- Program defined by '{psmname}.psm'.
--
-- Generated by KCPSM6 Assembler: 2016-03-03T15:40:09. 
--
-- Assembler used ROM_form template: ROM_form_JTAGLoader_14March13.vhd
--
-- Standard IEEE libraries
--
--
package jtag_loader_pkg is
 function addr_width_calc (size_in_k: integer) return integer;
end jtag_loader_pkg;
--
package body jtag_loader_pkg is
  function addr_width_calc (size_in_k: integer) return integer is
   begin
    if (size_in_k = 1) then return 10;
      elsif (size_in_k = 2) then return 11;
      elsif (size_in_k = 4) then return 12;
      else report "Invalid BlockRAM size. Please set to 1, 2 or 4 K words." severity FAILURE;
    end if;
    return 0;
  end function addr_width_calc;
end package body;
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.jtag_loader_pkg.ALL;
--
-- The Unisim Library is used to define Xilinx primitives. It is also used during
-- simulation. The source can be viewed at %XILINX%\vhdl\src\unisims\unisim_VCOMP.vhd
--  
library unisim;
use unisim.vcomponents.all;
--
--
entity m25p16_spi_uart_bridge is
  generic(             C_FAMILY : string := "S6"; 
              C_RAM_SIZE_KWORDS : integer := 1;
           C_JTAG_LOADER_ENABLE : integer := 0);
  Port (      address : in std_logic_vector(11 downto 0);
          instruction : out std_logic_vector(17 downto 0);
               enable : in std_logic;
                  rdl : out std_logic;                    
                  clk : in std_logic);
  end m25p16_spi_uart_bridge;
--
architecture low_level_definition of m25p16_spi_uart_bridge is
--
signal       address_a : std_logic_vector(15 downto 0);
signal        pipe_a11 : std_logic;
signal       data_in_a : std_logic_vector(35 downto 0);
signal      data_out_a : std_logic_vector(35 downto 0);
signal    data_out_a_l : std_logic_vector(35 downto 0);
signal    data_out_a_h : std_logic_vector(35 downto 0);
signal   data_out_a_ll : std_logic_vector(35 downto 0);
signal   data_out_a_lh : std_logic_vector(35 downto 0);
signal   data_out_a_hl : std_logic_vector(35 downto 0);
signal   data_out_a_hh : std_logic_vector(35 downto 0);
signal       address_b : std_logic_vector(15 downto 0);
signal       data_in_b : std_logic_vector(35 downto 0);
signal     data_in_b_l : std_logic_vector(35 downto 0);
signal    data_in_b_ll : std_logic_vector(35 downto 0);
signal    data_in_b_hl : std_logic_vector(35 downto 0);
signal      data_out_b : std_logic_vector(35 downto 0);
signal    data_out_b_l : std_logic_vector(35 downto 0);
signal   data_out_b_ll : std_logic_vector(35 downto 0);
signal   data_out_b_hl : std_logic_vector(35 downto 0);
signal     data_in_b_h : std_logic_vector(35 downto 0);
signal    data_in_b_lh : std_logic_vector(35 downto 0);
signal    data_in_b_hh : std_logic_vector(35 downto 0);
signal    data_out_b_h : std_logic_vector(35 downto 0);
signal   data_out_b_lh : std_logic_vector(35 downto 0);
signal   data_out_b_hh : std_logic_vector(35 downto 0);
signal        enable_b : std_logic;
signal           clk_b : std_logic;
signal            we_b : std_logic_vector(7 downto 0);
signal          we_b_l : std_logic_vector(3 downto 0);
signal          we_b_h : std_logic_vector(3 downto 0);
-- 
signal       jtag_addr : std_logic_vector(11 downto 0);
signal         jtag_we : std_logic;
signal       jtag_we_l : std_logic;
signal       jtag_we_h : std_logic;
signal        jtag_clk : std_logic;
signal        jtag_din : std_logic_vector(17 downto 0);
signal       jtag_dout : std_logic_vector(17 downto 0);
signal     jtag_dout_1 : std_logic_vector(17 downto 0);
signal         jtag_en : std_logic_vector(0 downto 0);
-- 
signal picoblaze_reset : std_logic_vector(0 downto 0);
signal         rdl_bus : std_logic_vector(0 downto 0);
--
constant BRAM_ADDRESS_WIDTH  : integer := addr_width_calc(C_RAM_SIZE_KWORDS);
--
--
component jtag_loader_6
generic(                C_JTAG_LOADER_ENABLE : integer := 1;
                                    C_FAMILY : string  := "V6";
                             C_NUM_PICOBLAZE : integer := 1;
                       C_BRAM_MAX_ADDR_WIDTH : integer := 10;
          C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                                C_JTAG_CHAIN : integer := 2;
                              C_ADDR_WIDTH_0 : integer := 10;
                              C_ADDR_WIDTH_1 : integer := 10;
                              C_ADDR_WIDTH_2 : integer := 10;
                              C_ADDR_WIDTH_3 : integer := 10;
                              C_ADDR_WIDTH_4 : integer := 10;
                              C_ADDR_WIDTH_5 : integer := 10;
                              C_ADDR_WIDTH_6 : integer := 10;
                              C_ADDR_WIDTH_7 : integer := 10);
port(              picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                           jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                          jtag_din : out STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                         jtag_addr : out STD_LOGIC_VECTOR(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
                          jtag_clk : out std_logic;
                           jtag_we : out std_logic;
                       jtag_dout_0 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_1 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_2 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_3 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_4 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_5 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_6 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
                       jtag_dout_7 : in STD_LOGIC_VECTOR(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end component;
--
begin
  --
  --  
  ram_1k_generate : if (C_RAM_SIZE_KWORDS = 1) generate
 
    s6: if (C_FAMILY = "S6") generate 
      --
      address_a(13 downto 0) <= address(9 downto 0) & "0000";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "0000000000000000000000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "0000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB16BWER
      generic map ( DATA_WIDTH_A => 18,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 18,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"08200056092000560056129F003C150CB001B03171FF0049B6740045B1740049",
                    INIT_01 => X"003C15526021D04090001700180019072017003C15662018F215F820D9200053",
                    INIT_02 => X"500035DFD000D57B9000D561203C150A003C150D202400E9037A00B303C403E6",
                    INIT_03 => X"D501603CD00490005000150A500095F690001507E03A9511900095E9900015B9",
                    INIT_04 => X"B10090010000204D108011961298204D1040110D1203204D1010112712005000",
                    INIT_05 => X"00571200B654500092046058D2019205D208B6645000B67400565000604DB200",
                    INIT_06 => X"005D580450000620007007200057005D2070005D500002700057028000570290",
                    INIT_07 => X"120000571200B654206E191F006E191E207002600057005D58045000B6740057",
                    INIT_08 => X"12FF005712FF0057120100571201005712A8005712C000571280005712040057",
                    INIT_09 => X"12030057120200571201005712DC005712080057120000571200005712FF0057",
                    INIT_0A => X"60ABC67016010057A2602129B6740057120200571201005712A8005712C00057",
                    INIT_0B => X"0057120000571200B654607CD280B20060B1D204006618001939203C15655000",
                    INIT_0C => X"190CB6540073B615B7141904006EB71318081900B67400AB1713160000571204",
                    INIT_0D => X"206E004117101901500061101000D1FFB11C0129B67400AB171C1616005D180C",
                    INIT_0E => X"182819031000D1FF700061107001206E0041170819010049B6740045B2740045",
                    INIT_0F => X"20DCD21B20DCD21A20DCD21820E5D21C182C215AD2172148D2002108D2140066",
                    INIT_10 => X"D2172325D2000066180819031000D100003C153F214FD21320DCD21E20DCD21D",
                    INIT_11 => X"20DCD21020DCD21E20DCD21D20DCD21B20DCD21A20DCD21820E0D21C180C2337",
                    INIT_12 => X"18481700007818280078180817085000003C15452128D2142335D215232CD213",
                    INIT_13 => X"190000DC182800DC1808007818E8007818C8007818A800781888007818680078",
                    INIT_14 => X"19012041006E1701182C1901003C154F20731650170019047100006E17011828",
                    INIT_15 => X"6FF06CC0700100681828192650007F007E007D007C007B007A00006E1702182C",
                    INIT_16 => X"0068182819280B700A601000F700D600219BF700D60073006168FFFFDC047000",
                    INIT_17 => X"DC04350014011301E2300057140015001300005712380057026000570270B654",
                    INIT_18 => X"192827900680B674095008406330617AE5B0C4A02189D3FF2189D20A2184FFFF",
                    INIT_19 => X"6990688067706330700150007000DFFF41A9DFFF7001006E174019010073182C",
                    INIT_1A => X"DF0061AAC4301401003CA54014006BA01000FB00DA0070006FF06CC06BB06AA0",
                    INIT_1B => X"61B060A050001FFE61BEF40AF50DD302B401B500500013001F0141B6DF0001FB",
                    INIT_1C => X"1B0321D4D2801415174003BB19071800170061DE02DD1AF61B02699068806770",
                    INIT_1D => X"6A007000222DE1D414011701622DC25021DED200A54003BB21DE622D02DE1A0B",
                    INIT_1E => X"90000030002A003CA5A01B0D1A07170050001E001D0070016990688067706B10",
                    INIT_1F => X"02DD1AE81B021C001D00500061EBCAB01A01475061F190014900480047061004",
                    INIT_20 => X"22071C026210D262A22D01E850001A5E1B02170018001906620AD220B2056222",
                    INIT_21 => X"1B021C03622DD27250001A5E1B021C05621CD27350001A5A1B021C036216D265",
                    INIT_22 => X"1A561B021C0050001A311B021C04A22D01E81C02622D02DD1AEE1B0250001A52",
                    INIT_23 => X"1E005000225E7000FE1FDDFF3E001D0170010041623FD3007000633070015000",
                    INIT_24 => X"1D0003F822402240DE0F224ECE301E01700039003800170103D570016240A4E0",
                    INIT_25 => X"6CC07001700003F803E67001225E170018FE19064BA01A531B02025E50001E00",
                    INIT_26 => X"02BA70001C00170018FF19066D706B806A907001006818281924227ADC007000",
                    INIT_27 => X"02BA2283DC050068182819242281DC032281DC041D006CC067D068B069A002CB",
                    INIT_28 => X"6295D902229400DC00E56294D100180C0180628ED9010290700120E5182C02CB",
                    INIT_29 => X"1903F20A006618081900F109F80003B01300F800588018007001036C70006380",
                    INIT_2A => X"026000570270B654700068202280700062A3D80C180102AAA2801800F20B0066",
                    INIT_2B => X"3800170103BB1440151F70015000700137001601B67400570280005712340057",
                    INIT_2C => X"172019010073182C19245000700062BDB500940102AA22C9D20022C5DC023900",
                    INIT_2D => X"D5004BA01400500062D1F720D6000068182819205000D217006618281903006E",
                    INIT_2E => X"154F15501500152F152015541545154722DE3B001A0114015000C560A6401000",
                    INIT_2F => X"15741561157A15691572156F15681574157515411500155F152F152015541553",
                    INIT_30 => X"156115741552155715591500156315691573156115421520153A156E156F1569",
                    INIT_31 => X"80263E001D0170011500153D1534155715611574155215571559153615341557",
                    INIT_32 => X"180C1901031C752B206E1701180C19016041031C752E50007000003CFE03DDFF",
                    INIT_33 => X"5000D20292059000F710D600006818081926231C752A2041006E700067107001",
                    INIT_34 => X"16007001005712180057026000570270B6546460657000681808192867706660",
                    INIT_35 => X"64407001B6746354B7009601D20292046355D2039205D2087000677066601710",
                    INIT_36 => X"120000571200B634206E174019010073180C1928275006407000677066606550",
                    INIT_37 => X"00571201B6340045036C130023ABB67400575202023000571200005712600057",
                    INIT_38 => X"12000057120000571203B6340045B674005712200057120C0057120000571200",
                    INIT_39 => X"005712410057121F0057120900571202B6340045B674005712FF0057127F0057",
                    INIT_3A => X"5000B6740045B6F4004503B0131D63ABF30ED280B31DB2000045B6740057122A",
                    INIT_3B => X"028000560290005612035000B67463B3C3401301E2300057A23014090430B634",
                    INIT_3C => X"D3FFE23000561300005602700056028000560290005612032053005602700056",
                    INIT_3D => X"02D00056027000560280005602900056120203F60D205000B67423CD130123D3",
                    INIT_3E => X"03F3005302700056028000560290005612D803F6500003F863E1D20103F30053",
                    INIT_3F => X"0000000000000000000000002053120420531206205300561205500063EFD201",
                   INITP_00 => X"22888A2A888AA8888A30AAAD52020202B099DDDD8DD88AAA8C00A3562288A6AA",
                   INITP_01 => X"DDDD37760DDE82AAA09D2A0828080A0888B4D822D62A22222222222222222222",
                   INITP_02 => X"55A022228035D5D75E095560280880602222222208828DDDDDDDDDD3760D8DDD",
                   INITP_03 => X"802D4D54EA0083557B5DD2B83420381558D508777582D75555FDDE0816075DDD",
                   INITP_04 => X"B60DD156B00578377EB808282B35D5B42F55EDDE02038E080D80D80D8DE800D3",
                   INITP_05 => X"AAAAAAAAAAAA95D362D60D82082F5B75583B5A2222DBD60A282A20EDDAB434E2",
                   INITP_06 => X"8A8A8222228205D57B58C2D438889605C35826B70980E6F1D7AAAAAAAAAAAAAA",
                   INITP_07 => X"00088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA22222A8888")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a(31 downto 0),
                  DOPA => data_out_a(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b(31 downto 0),
                  DOPB => data_out_b(35 downto 32), 
                   DIB => data_in_b(31 downto 0),
                  DIPB => data_in_b(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --               
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"08200056092000560056129F003C150CB001B03171FF0049B6740045B1740049",
                    INIT_01 => X"003C15526021D04090001700180019072017003C15662018F215F820D9200053",
                    INIT_02 => X"500035DFD000D57B9000D561203C150A003C150D202400E9037A00B303C403E6",
                    INIT_03 => X"D501603CD00490005000150A500095F690001507E03A9511900095E9900015B9",
                    INIT_04 => X"B10090010000204D108011961298204D1040110D1203204D1010112712005000",
                    INIT_05 => X"00571200B654500092046058D2019205D208B6645000B67400565000604DB200",
                    INIT_06 => X"005D580450000620007007200057005D2070005D500002700057028000570290",
                    INIT_07 => X"120000571200B654206E191F006E191E207002600057005D58045000B6740057",
                    INIT_08 => X"12FF005712FF0057120100571201005712A8005712C000571280005712040057",
                    INIT_09 => X"12030057120200571201005712DC005712080057120000571200005712FF0057",
                    INIT_0A => X"60ABC67016010057A2602129B6740057120200571201005712A8005712C00057",
                    INIT_0B => X"0057120000571200B654607CD280B20060B1D204006618001939203C15655000",
                    INIT_0C => X"190CB6540073B615B7141904006EB71318081900B67400AB1713160000571204",
                    INIT_0D => X"206E004117101901500061101000D1FFB11C0129B67400AB171C1616005D180C",
                    INIT_0E => X"182819031000D1FF700061107001206E0041170819010049B6740045B2740045",
                    INIT_0F => X"20DCD21B20DCD21A20DCD21820E5D21C182C215AD2172148D2002108D2140066",
                    INIT_10 => X"D2172325D2000066180819031000D100003C153F214FD21320DCD21E20DCD21D",
                    INIT_11 => X"20DCD21020DCD21E20DCD21D20DCD21B20DCD21A20DCD21820E0D21C180C2337",
                    INIT_12 => X"18481700007818280078180817085000003C15452128D2142335D215232CD213",
                    INIT_13 => X"190000DC182800DC1808007818E8007818C8007818A800781888007818680078",
                    INIT_14 => X"19012041006E1701182C1901003C154F20731650170019047100006E17011828",
                    INIT_15 => X"6FF06CC0700100681828192650007F007E007D007C007B007A00006E1702182C",
                    INIT_16 => X"0068182819280B700A601000F700D600219BF700D60073006168FFFFDC047000",
                    INIT_17 => X"DC04350014011301E2300057140015001300005712380057026000570270B654",
                    INIT_18 => X"192827900680B674095008406330617AE5B0C4A02189D3FF2189D20A2184FFFF",
                    INIT_19 => X"6990688067706330700150007000DFFF41A9DFFF7001006E174019010073182C",
                    INIT_1A => X"DF0061AAC4301401003CA54014006BA01000FB00DA0070006FF06CC06BB06AA0",
                    INIT_1B => X"61B060A050001FFE61BEF40AF50DD302B401B500500013001F0141B6DF0001FB",
                    INIT_1C => X"1B0321D4D2801415174003BB19071800170061DE02DD1AF61B02699068806770",
                    INIT_1D => X"6A007000222DE1D414011701622DC25021DED200A54003BB21DE622D02DE1A0B",
                    INIT_1E => X"90000030002A003CA5A01B0D1A07170050001E001D0070016990688067706B10",
                    INIT_1F => X"02DD1AE81B021C001D00500061EBCAB01A01475061F190014900480047061004",
                    INIT_20 => X"22071C026210D262A22D01E850001A5E1B02170018001906620AD220B2056222",
                    INIT_21 => X"1B021C03622DD27250001A5E1B021C05621CD27350001A5A1B021C036216D265",
                    INIT_22 => X"1A561B021C0050001A311B021C04A22D01E81C02622D02DD1AEE1B0250001A52",
                    INIT_23 => X"1E005000225E7000FE1FDDFF3E001D0170010041623FD3007000633070015000",
                    INIT_24 => X"1D0003F822402240DE0F224ECE301E01700039003800170103D570016240A4E0",
                    INIT_25 => X"6CC07001700003F803E67001225E170018FE19064BA01A531B02025E50001E00",
                    INIT_26 => X"02BA70001C00170018FF19066D706B806A907001006818281924227ADC007000",
                    INIT_27 => X"02BA2283DC050068182819242281DC032281DC041D006CC067D068B069A002CB",
                    INIT_28 => X"6295D902229400DC00E56294D100180C0180628ED9010290700120E5182C02CB",
                    INIT_29 => X"1903F20A006618081900F109F80003B01300F800588018007001036C70006380",
                    INIT_2A => X"026000570270B654700068202280700062A3D80C180102AAA2801800F20B0066",
                    INIT_2B => X"3800170103BB1440151F70015000700137001601B67400570280005712340057",
                    INIT_2C => X"172019010073182C19245000700062BDB500940102AA22C9D20022C5DC023900",
                    INIT_2D => X"D5004BA01400500062D1F720D6000068182819205000D217006618281903006E",
                    INIT_2E => X"154F15501500152F152015541545154722DE3B001A0114015000C560A6401000",
                    INIT_2F => X"15741561157A15691572156F15681574157515411500155F152F152015541553",
                    INIT_30 => X"156115741552155715591500156315691573156115421520153A156E156F1569",
                    INIT_31 => X"80263E001D0170011500153D1534155715611574155215571559153615341557",
                    INIT_32 => X"180C1901031C752B206E1701180C19016041031C752E50007000003CFE03DDFF",
                    INIT_33 => X"5000D20292059000F710D600006818081926231C752A2041006E700067107001",
                    INIT_34 => X"16007001005712180057026000570270B6546460657000681808192867706660",
                    INIT_35 => X"64407001B6746354B7009601D20292046355D2039205D2087000677066601710",
                    INIT_36 => X"120000571200B634206E174019010073180C1928275006407000677066606550",
                    INIT_37 => X"00571201B6340045036C130023ABB67400575202023000571200005712600057",
                    INIT_38 => X"12000057120000571203B6340045B674005712200057120C0057120000571200",
                    INIT_39 => X"005712410057121F0057120900571202B6340045B674005712FF0057127F0057",
                    INIT_3A => X"5000B6740045B6F4004503B0131D63ABF30ED280B31DB2000045B6740057122A",
                    INIT_3B => X"028000560290005612035000B67463B3C3401301E2300057A23014090430B634",
                    INIT_3C => X"D3FFE23000561300005602700056028000560290005612032053005602700056",
                    INIT_3D => X"02D00056027000560280005602900056120203F60D205000B67423CD130123D3",
                    INIT_3E => X"03F3005302700056028000560290005612D803F6500003F863E1D20103F30053",
                    INIT_3F => X"0000000000000000000000002053120420531206205300561205500063EFD201",
                   INITP_00 => X"22888A2A888AA8888A30AAAD52020202B099DDDD8DD88AAA8C00A3562288A6AA",
                   INITP_01 => X"DDDD37760DDE82AAA09D2A0828080A0888B4D822D62A22222222222222222222",
                   INITP_02 => X"55A022228035D5D75E095560280880602222222208828DDDDDDDDDD3760D8DDD",
                   INITP_03 => X"802D4D54EA0083557B5DD2B83420381558D508777582D75555FDDE0816075DDD",
                   INITP_04 => X"B60DD156B00578377EB808282B35D5B42F55EDDE02038E080D80D80D8DE800D3",
                   INITP_05 => X"AAAAAAAAAAAA95D362D60D82082F5B75583B5A2222DBD60A282A20EDDAB434E2",
                   INITP_06 => X"8A8A8222228205D57B58C2D438889605C35826B70980E6F1D7AAAAAAAAAAAAAA",
                   INITP_07 => X"00088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA22222A8888")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a(13 downto 0) <= address(9 downto 0) & "1111";
      instruction <= data_out_a(17 downto 0);
      data_in_a(17 downto 0) <= "0000000000000000" & address(11 downto 10);
      jtag_dout <= data_out_b(17 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b(17 downto 0) <= data_out_b(17 downto 0);
        address_b(13 downto 0) <= "11111111111111";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b(17 downto 0) <= jtag_din(17 downto 0);
        address_b(13 downto 0) <= jtag_addr(9 downto 0) & "1111";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      -- 
      kcpsm6_rom: RAMB18E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => "000000000000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"08200056092000560056129F003C150CB001B03171FF0049B6740045B1740049",
                    INIT_01 => X"003C15526021D04090001700180019072017003C15662018F215F820D9200053",
                    INIT_02 => X"500035DFD000D57B9000D561203C150A003C150D202400E9037A00B303C403E6",
                    INIT_03 => X"D501603CD00490005000150A500095F690001507E03A9511900095E9900015B9",
                    INIT_04 => X"B10090010000204D108011961298204D1040110D1203204D1010112712005000",
                    INIT_05 => X"00571200B654500092046058D2019205D208B6645000B67400565000604DB200",
                    INIT_06 => X"005D580450000620007007200057005D2070005D500002700057028000570290",
                    INIT_07 => X"120000571200B654206E191F006E191E207002600057005D58045000B6740057",
                    INIT_08 => X"12FF005712FF0057120100571201005712A8005712C000571280005712040057",
                    INIT_09 => X"12030057120200571201005712DC005712080057120000571200005712FF0057",
                    INIT_0A => X"60ABC67016010057A2602129B6740057120200571201005712A8005712C00057",
                    INIT_0B => X"0057120000571200B654607CD280B20060B1D204006618001939203C15655000",
                    INIT_0C => X"190CB6540073B615B7141904006EB71318081900B67400AB1713160000571204",
                    INIT_0D => X"206E004117101901500061101000D1FFB11C0129B67400AB171C1616005D180C",
                    INIT_0E => X"182819031000D1FF700061107001206E0041170819010049B6740045B2740045",
                    INIT_0F => X"20DCD21B20DCD21A20DCD21820E5D21C182C215AD2172148D2002108D2140066",
                    INIT_10 => X"D2172325D2000066180819031000D100003C153F214FD21320DCD21E20DCD21D",
                    INIT_11 => X"20DCD21020DCD21E20DCD21D20DCD21B20DCD21A20DCD21820E0D21C180C2337",
                    INIT_12 => X"18481700007818280078180817085000003C15452128D2142335D215232CD213",
                    INIT_13 => X"190000DC182800DC1808007818E8007818C8007818A800781888007818680078",
                    INIT_14 => X"19012041006E1701182C1901003C154F20731650170019047100006E17011828",
                    INIT_15 => X"6FF06CC0700100681828192650007F007E007D007C007B007A00006E1702182C",
                    INIT_16 => X"0068182819280B700A601000F700D600219BF700D60073006168FFFFDC047000",
                    INIT_17 => X"DC04350014011301E2300057140015001300005712380057026000570270B654",
                    INIT_18 => X"192827900680B674095008406330617AE5B0C4A02189D3FF2189D20A2184FFFF",
                    INIT_19 => X"6990688067706330700150007000DFFF41A9DFFF7001006E174019010073182C",
                    INIT_1A => X"DF0061AAC4301401003CA54014006BA01000FB00DA0070006FF06CC06BB06AA0",
                    INIT_1B => X"61B060A050001FFE61BEF40AF50DD302B401B500500013001F0141B6DF0001FB",
                    INIT_1C => X"1B0321D4D2801415174003BB19071800170061DE02DD1AF61B02699068806770",
                    INIT_1D => X"6A007000222DE1D414011701622DC25021DED200A54003BB21DE622D02DE1A0B",
                    INIT_1E => X"90000030002A003CA5A01B0D1A07170050001E001D0070016990688067706B10",
                    INIT_1F => X"02DD1AE81B021C001D00500061EBCAB01A01475061F190014900480047061004",
                    INIT_20 => X"22071C026210D262A22D01E850001A5E1B02170018001906620AD220B2056222",
                    INIT_21 => X"1B021C03622DD27250001A5E1B021C05621CD27350001A5A1B021C036216D265",
                    INIT_22 => X"1A561B021C0050001A311B021C04A22D01E81C02622D02DD1AEE1B0250001A52",
                    INIT_23 => X"1E005000225E7000FE1FDDFF3E001D0170010041623FD3007000633070015000",
                    INIT_24 => X"1D0003F822402240DE0F224ECE301E01700039003800170103D570016240A4E0",
                    INIT_25 => X"6CC07001700003F803E67001225E170018FE19064BA01A531B02025E50001E00",
                    INIT_26 => X"02BA70001C00170018FF19066D706B806A907001006818281924227ADC007000",
                    INIT_27 => X"02BA2283DC050068182819242281DC032281DC041D006CC067D068B069A002CB",
                    INIT_28 => X"6295D902229400DC00E56294D100180C0180628ED9010290700120E5182C02CB",
                    INIT_29 => X"1903F20A006618081900F109F80003B01300F800588018007001036C70006380",
                    INIT_2A => X"026000570270B654700068202280700062A3D80C180102AAA2801800F20B0066",
                    INIT_2B => X"3800170103BB1440151F70015000700137001601B67400570280005712340057",
                    INIT_2C => X"172019010073182C19245000700062BDB500940102AA22C9D20022C5DC023900",
                    INIT_2D => X"D5004BA01400500062D1F720D6000068182819205000D217006618281903006E",
                    INIT_2E => X"154F15501500152F152015541545154722DE3B001A0114015000C560A6401000",
                    INIT_2F => X"15741561157A15691572156F15681574157515411500155F152F152015541553",
                    INIT_30 => X"156115741552155715591500156315691573156115421520153A156E156F1569",
                    INIT_31 => X"80263E001D0170011500153D1534155715611574155215571559153615341557",
                    INIT_32 => X"180C1901031C752B206E1701180C19016041031C752E50007000003CFE03DDFF",
                    INIT_33 => X"5000D20292059000F710D600006818081926231C752A2041006E700067107001",
                    INIT_34 => X"16007001005712180057026000570270B6546460657000681808192867706660",
                    INIT_35 => X"64407001B6746354B7009601D20292046355D2039205D2087000677066601710",
                    INIT_36 => X"120000571200B634206E174019010073180C1928275006407000677066606550",
                    INIT_37 => X"00571201B6340045036C130023ABB67400575202023000571200005712600057",
                    INIT_38 => X"12000057120000571203B6340045B674005712200057120C0057120000571200",
                    INIT_39 => X"005712410057121F0057120900571202B6340045B674005712FF0057127F0057",
                    INIT_3A => X"5000B6740045B6F4004503B0131D63ABF30ED280B31DB2000045B6740057122A",
                    INIT_3B => X"028000560290005612035000B67463B3C3401301E2300057A23014090430B634",
                    INIT_3C => X"D3FFE23000561300005602700056028000560290005612032053005602700056",
                    INIT_3D => X"02D00056027000560280005602900056120203F60D205000B67423CD130123D3",
                    INIT_3E => X"03F3005302700056028000560290005612D803F6500003F863E1D20103F30053",
                    INIT_3F => X"0000000000000000000000002053120420531206205300561205500063EFD201",
                   INITP_00 => X"22888A2A888AA8888A30AAAD52020202B099DDDD8DD88AAA8C00A3562288A6AA",
                   INITP_01 => X"DDDD37760DDE82AAA09D2A0828080A0888B4D822D62A22222222222222222222",
                   INITP_02 => X"55A022228035D5D75E095560280880602222222208828DDDDDDDDDD3760D8DDD",
                   INITP_03 => X"802D4D54EA0083557B5DD2B83420381558D508777582D75555FDDE0816075DDD",
                   INITP_04 => X"B60DD156B00578377EB808282B35D5B42F55EDDE02038E080D80D80D8DE800D3",
                   INITP_05 => X"AAAAAAAAAAAA95D362D60D82082F5B75583B5A2222DBD60A282A20EDDAB434E2",
                   INITP_06 => X"8A8A8222228205D57B58C2D438889605C35826B70980E6F1D7AAAAAAAAAAAAAA",
                   INITP_07 => X"00088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA22222A8888")
      port map(   ADDRARDADDR => address_a(13 downto 0),
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(15 downto 0),
                      DOPADOP => data_out_a(17 downto 16), 
                        DIADI => data_in_a(15 downto 0),
                      DIPADIP => data_in_a(17 downto 16), 
                          WEA => "00",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b(13 downto 0),
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(15 downto 0),
                      DOPBDOP => data_out_b(17 downto 16), 
                        DIBDI => data_in_b(15 downto 0),
                      DIPBDIP => data_in_b(17 downto 16), 
                        WEBWE => we_b(3 downto 0),
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0');
      --
    end generate akv7;
    --
  end generate ram_1k_generate;
  --
  --
  --
  ram_2k_generate : if (C_RAM_SIZE_KWORDS = 2) generate
    --
    --
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        we_b(3 downto 0) <= jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"3C52214000000007173C66181520205320562056569F3C0C0131FF4974457449",
                    INIT_01 => X"013C0400000A00F600073A1100E900B900DF007B00613C0A3C0D24E97AB3C4E6",
                    INIT_02 => X"57005400045801050864007456004D000001004D8096984D400D034D10270000",
                    INIT_03 => X"005700546E1F6E1E7060575D040074575D0400207020575D705D007057805790",
                    INIT_04 => X"035702570157DC57085700570057FF57FF57FF5701570157A857C05780570457",
                    INIT_05 => X"57005700547C8000B1046600393C6500AB7001576029745702570157A857C057",
                    INIT_06 => X"6E411001001000FF1C2974AB1C165D0C0C54731514046E13080074AB13005704",
                    INIT_07 => X"DC1BDC1ADC18E51C2C5A174800081466280300FF0010016E4108014974457445",
                    INIT_08 => X"DC10DC1EDC1DDC1BDC1ADC18E01C0C3717250066080300003C3F4F13DC1EDC1D",
                    INIT_09 => X"00DC28DC0878E878C878A8788878687848007828780808003C45281435152C13",
                    INIT_0A => X"F0C001682826000000000000006E022C01416E012C013C4F73500004006E0128",
                    INIT_0B => X"0400010130570000005738576057705468282870600000009B00000068FF0400",
                    INIT_0C => X"90807030010000FFA9FF016E4001732C289080745040307AB0A089FF890A84FF",
                    INIT_0D => X"B0A000FEBE0A0D020100000001B600FB00AA30013C4000A000000000F0C0B0A0",
                    INIT_0E => X"00002DD401012D50DE0040BBDE2DDE0B03D4801540BB070000DEDDF602908070",
                    INIT_0F => X"DDE802000000EBB00150F1010000060400302A3CA00D07000000000190807010",
                    INIT_10 => X"02032D72005E02051C73005A02031665070210622DE8005E020000060A200522",
                    INIT_11 => X"00005E001FFF000101413F0000300100560200003102042DE8022DDDEE020052",
                    INIT_12 => X"C00100F8E6015E00FE06A053025E000000F840400F4E300100000001D50140E0",
                    INIT_13 => X"BA83056828248103810400C0D0B0A0CBBA000000FF06708090016828247A0000",
                    INIT_14 => X"030A6608000900B000008000016C0080950294DCE594000C808E019001E52CCB",
                    INIT_15 => X"0001BB401F01000100017457805734576057705400208000A30C01AA80000B66",
                    INIT_16 => X"00A00000D1200068282000176628036E2001732C240000BD0001AAC900C50200",
                    INIT_17 => X"74617A69726F68747541005F2F2054534F50002F20544547DE00010100604000",
                    INIT_18 => X"26000101003D345761745257593634576174525759006369736142203A6E6F69",
                    INIT_19 => X"0002050010006808261C2A416E0010010C011C2B6E010C01411C2E00003C03FF",
                    INIT_1A => X"4001745400010204550305080070601000015718576057705460706808287060",
                    INIT_1B => X"570134456C00AB745702305700576057005700346E4001730C28504000706050",
                    INIT_1C => X"5741571F5709570234457457FF577F5700570057033445745720570C57005700",
                    INIT_1D => X"80569056030074B34001305730093034007445F445B01DAB0E801D004574572A",
                    INIT_1E => X"D05670568056905602F6200074CD01D3FF305600567056805690560353567056",
                    INIT_1F => X"0000000000005304530653560500EF01F353705680569056D8F600F8E101F353",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0054546035C88D48000A04000000000005000400000084448555554B45222122",
                   INITP_01 => X"2A6A0D2B14B8CECD9B5FC54AB1CCCBBB518032DC8552953A8000426A00014560",
                   INITP_02 => X"FFFFFF44C442C881688000008D8542A0042A17081968C05805144484820804D0",
                   INITP_03 => X"0002805A0067900001C007A0000000000E0006651885002608E275612FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_l(31 downto 0),
                  DOPA => data_out_a_l(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_l(31 downto 0),
                  DOPB => data_out_b_l(35 downto 32), 
                   DIB => data_in_b_l(31 downto 0),
                  DIPB => data_in_b_l(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_h: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"000AB068480B0C0C10000A90F9FCEC00040004000009000A5858B8005B005800",
                    INIT_01 => X"6AB06848288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100A000A100001000101",
                    INIT_02 => X"00095B2849B06949695B285B0028B0D9D8C80010080809100808091008080928",
                    INIT_03 => X"0900095B100C000C100100002C285B00002C2803000300001000280100010001",
                    INIT_04 => X"0900090009000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"000900095BB0E959B0E9000C0C100A28B0E38B0051105B000900090009000900",
                    INIT_06 => X"10000B0C28B088E858005B000B0B000C0C5B005B5B0C005B0C0C5B000B0B0009",
                    INIT_07 => X"90E990E990E990E90C90E990E990E9000C0C88E8B8B0B810000B0C005B005900",
                    INIT_08 => X"90E990E990E990E990E990E990E90C91E991E9000C0C88E8000A90E990E990E9",
                    INIT_09 => X"0C000C000C000C000C000C000C000C000C0B000C000C0B28000A90E991E991E9",
                    INIT_0A => X"B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C000A100B0B0CB8000B0C",
                    INIT_0B => X"EE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8",
                    INIT_0C => X"B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FF",
                    INIT_0D => X"B0B0280FB0FAFAE95A5A28098FA0EF80EFB0E28A00520A3588FDEDB8B7B6B5B5",
                    INIT_0E => X"B5B811F08A8BB1E190E9520110B1010D0D90E90A0B010C0C0BB0010D0DB4B4B3",
                    INIT_0F => X"010D0D0E0E28B0E58D23B0C8A4A4A308C8000000520D0D0B280F0EB8B4B4B3B5",
                    INIT_10 => X"0D0EB1E9280D0D0EB1E9280D0D0EB1E9110EB1E9D100280D0D0B0C0CB1E959B1",
                    INIT_11 => X"0F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0E280D0D0ED1000EB1010D0D280D",
                    INIT_12 => X"B6B8B80101B8110B0C0C250D0D01280F0E0111916F91E78FB89C9C8B01B8B152",
                    INIT_13 => X"0191EE000C0C91EE91EE0EB6B3B4B40101B80E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_14 => X"0C79000C0C787C01097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_15 => X"9C8B010A0AB828B89B8B5B00010009000100015BB8B411B8B1EC8C01510C7900",
                    INIT_16 => X"EA250A28B1FBEB000C0CA8E9000C0C000B0C000C0C28B8B1DACA0191E991EE9C",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388",
                    INIT_18 => X"C09F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_19 => X"A86949C8FBEB000C0C11BA1000B8B3B80C0C01BA100B0C0CB001BA28B8807FEE",
                    INIT_1A => X"B2B85BB1DBCB6949B1694969B8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1B => X"00095B000109115B00290100090009000900095B100B0C000C0C9383B8B3B3B2",
                    INIT_1C => X"00090009000900095B005B000900090009000900095B005B0009000900090009",
                    INIT_1D => X"0100010009285BB1E1897100518A025B285B005B000109B1F9E95959005B0009",
                    INIT_1E => X"0100010001000100090106285B118991E9710009000100010001000910000100",
                    INIT_1F => X"0000000000001009100910000928B169010001000100010009012801B1690100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"AA552B9FCA726232ACA59755555555555AB7ABEAB4FE1111CAAAAABFA0D15ADF",
                   INITP_01 => X"8620F090729E4460282549900EB2112A0C55848932046284555529AAAAA952AA",
                   INITP_02 => X"FFFFFF895929273427355B93674EBC4DD281C0657E26748C70EB11B228A2AE09",
                   INITP_03 => X"02B6D57B555D6AAD5731FD0EAAF557AABB95590872986A90925D28DC9FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_h(31 downto 0),
                  DOPA => data_out_a_h(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_h(31 downto 0),
                  DOPB => data_out_b_h(35 downto 32), 
                   DIB => data_in_b_h(31 downto 0),
                  DIPB => data_in_b_h(35 downto 32), 
                   WEB => we_b(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"08200056092000560056129F003C150CB001B03171FF0049B6740045B1740049",
                    INIT_01 => X"003C15526021D04090001700180019072017003C15662018F215F820D9200053",
                    INIT_02 => X"500035DFD000D57B9000D561203C150A003C150D202400E9037A00B303C403E6",
                    INIT_03 => X"D501603CD00490005000150A500095F690001507E03A9511900095E9900015B9",
                    INIT_04 => X"B10090010000204D108011961298204D1040110D1203204D1010112712005000",
                    INIT_05 => X"00571200B654500092046058D2019205D208B6645000B67400565000604DB200",
                    INIT_06 => X"005D580450000620007007200057005D2070005D500002700057028000570290",
                    INIT_07 => X"120000571200B654206E191F006E191E207002600057005D58045000B6740057",
                    INIT_08 => X"12FF005712FF0057120100571201005712A8005712C000571280005712040057",
                    INIT_09 => X"12030057120200571201005712DC005712080057120000571200005712FF0057",
                    INIT_0A => X"60ABC67016010057A2602129B6740057120200571201005712A8005712C00057",
                    INIT_0B => X"0057120000571200B654607CD280B20060B1D204006618001939203C15655000",
                    INIT_0C => X"190CB6540073B615B7141904006EB71318081900B67400AB1713160000571204",
                    INIT_0D => X"206E004117101901500061101000D1FFB11C0129B67400AB171C1616005D180C",
                    INIT_0E => X"182819031000D1FF700061107001206E0041170819010049B6740045B2740045",
                    INIT_0F => X"20DCD21B20DCD21A20DCD21820E5D21C182C215AD2172148D2002108D2140066",
                    INIT_10 => X"D2172325D2000066180819031000D100003C153F214FD21320DCD21E20DCD21D",
                    INIT_11 => X"20DCD21020DCD21E20DCD21D20DCD21B20DCD21A20DCD21820E0D21C180C2337",
                    INIT_12 => X"18481700007818280078180817085000003C15452128D2142335D215232CD213",
                    INIT_13 => X"190000DC182800DC1808007818E8007818C8007818A800781888007818680078",
                    INIT_14 => X"19012041006E1701182C1901003C154F20731650170019047100006E17011828",
                    INIT_15 => X"6FF06CC0700100681828192650007F007E007D007C007B007A00006E1702182C",
                    INIT_16 => X"0068182819280B700A601000F700D600219BF700D60073006168FFFFDC047000",
                    INIT_17 => X"DC04350014011301E2300057140015001300005712380057026000570270B654",
                    INIT_18 => X"192827900680B674095008406330617AE5B0C4A02189D3FF2189D20A2184FFFF",
                    INIT_19 => X"6990688067706330700150007000DFFF41A9DFFF7001006E174019010073182C",
                    INIT_1A => X"DF0061AAC4301401003CA54014006BA01000FB00DA0070006FF06CC06BB06AA0",
                    INIT_1B => X"61B060A050001FFE61BEF40AF50DD302B401B500500013001F0141B6DF0001FB",
                    INIT_1C => X"1B0321D4D2801415174003BB19071800170061DE02DD1AF61B02699068806770",
                    INIT_1D => X"6A007000222DE1D414011701622DC25021DED200A54003BB21DE622D02DE1A0B",
                    INIT_1E => X"90000030002A003CA5A01B0D1A07170050001E001D0070016990688067706B10",
                    INIT_1F => X"02DD1AE81B021C001D00500061EBCAB01A01475061F190014900480047061004",
                    INIT_20 => X"22071C026210D262A22D01E850001A5E1B02170018001906620AD220B2056222",
                    INIT_21 => X"1B021C03622DD27250001A5E1B021C05621CD27350001A5A1B021C036216D265",
                    INIT_22 => X"1A561B021C0050001A311B021C04A22D01E81C02622D02DD1AEE1B0250001A52",
                    INIT_23 => X"1E005000225E7000FE1FDDFF3E001D0170010041623FD3007000633070015000",
                    INIT_24 => X"1D0003F822402240DE0F224ECE301E01700039003800170103D570016240A4E0",
                    INIT_25 => X"6CC07001700003F803E67001225E170018FE19064BA01A531B02025E50001E00",
                    INIT_26 => X"02BA70001C00170018FF19066D706B806A907001006818281924227ADC007000",
                    INIT_27 => X"02BA2283DC050068182819242281DC032281DC041D006CC067D068B069A002CB",
                    INIT_28 => X"6295D902229400DC00E56294D100180C0180628ED9010290700120E5182C02CB",
                    INIT_29 => X"1903F20A006618081900F109F80003B01300F800588018007001036C70006380",
                    INIT_2A => X"026000570270B654700068202280700062A3D80C180102AAA2801800F20B0066",
                    INIT_2B => X"3800170103BB1440151F70015000700137001601B67400570280005712340057",
                    INIT_2C => X"172019010073182C19245000700062BDB500940102AA22C9D20022C5DC023900",
                    INIT_2D => X"D5004BA01400500062D1F720D6000068182819205000D217006618281903006E",
                    INIT_2E => X"154F15501500152F152015541545154722DE3B001A0114015000C560A6401000",
                    INIT_2F => X"15741561157A15691572156F15681574157515411500155F152F152015541553",
                    INIT_30 => X"156115741552155715591500156315691573156115421520153A156E156F1569",
                    INIT_31 => X"80263E001D0170011500153D1534155715611574155215571559153615341557",
                    INIT_32 => X"180C1901031C752B206E1701180C19016041031C752E50007000003CFE03DDFF",
                    INIT_33 => X"5000D20292059000F710D600006818081926231C752A2041006E700067107001",
                    INIT_34 => X"16007001005712180057026000570270B6546460657000681808192867706660",
                    INIT_35 => X"64407001B6746354B7009601D20292046355D2039205D2087000677066601710",
                    INIT_36 => X"120000571200B634206E174019010073180C1928275006407000677066606550",
                    INIT_37 => X"00571201B6340045036C130023ABB67400575202023000571200005712600057",
                    INIT_38 => X"12000057120000571203B6340045B674005712200057120C0057120000571200",
                    INIT_39 => X"005712410057121F0057120900571202B6340045B674005712FF0057127F0057",
                    INIT_3A => X"5000B6740045B6F4004503B0131D63ABF30ED280B31DB2000045B6740057122A",
                    INIT_3B => X"028000560290005612035000B67463B3C3401301E2300057A23014090430B634",
                    INIT_3C => X"D3FFE23000561300005602700056028000560290005612032053005602700056",
                    INIT_3D => X"02D00056027000560280005602900056120203F60D205000B67423CD130123D3",
                    INIT_3E => X"03F3005302700056028000560290005612D803F6500003F863E1D20103F30053",
                    INIT_3F => X"0000000000000000000000002053120420531206205300561205500063EFD201",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"22888A2A888AA8888A30AAAD52020202B099DDDD8DD88AAA8C00A3562288A6AA",
                   INITP_01 => X"DDDD37760DDE82AAA09D2A0828080A0888B4D822D62A22222222222222222222",
                   INITP_02 => X"55A022228035D5D75E095560280880602222222208828DDDDDDDDDD3760D8DDD",
                   INITP_03 => X"802D4D54EA0083557B5DD2B83420381558D508777582D75555FDDE0816075DDD",
                   INITP_04 => X"B60DD156B00578377EB808282B35D5B42F55EDDE02038E080D80D80D8DE800D3",
                   INITP_05 => X"AAAAAAAAAAAA95D362D60D82082F5B75583B5A2222DBD60A282A20EDDAB434E2",
                   INITP_06 => X"8A8A8222228205D57B58C2D438889605C35826B70980E6F1D7AAAAAAAAAAAAAA",
                   INITP_07 => X"00088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA22222A8888",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(10 downto 0) & "1111";
      instruction <= data_out_a(33 downto 32) & data_out_a(15 downto 0);
      data_in_a <= "00000000000000000000000000000000000" & address(11);
      jtag_dout <= data_out_b(33 downto 32) & data_out_b(15 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b <= "00" & data_out_b(33 downto 32) & "0000000000000000" & data_out_b(15 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b <= "00" & jtag_din(17 downto 16) & "0000000000000000" & jtag_din(15 downto 0);
        address_b <= '1' & jtag_addr(10 downto 0) & "1111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom: RAMB36E1
      generic map ( READ_WIDTH_A => 18,
                    WRITE_WIDTH_A => 18,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 18,
                    WRITE_WIDTH_B => 18,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"08200056092000560056129F003C150CB001B03171FF0049B6740045B1740049",
                    INIT_01 => X"003C15526021D04090001700180019072017003C15662018F215F820D9200053",
                    INIT_02 => X"500035DFD000D57B9000D561203C150A003C150D202400E9037A00B303C403E6",
                    INIT_03 => X"D501603CD00490005000150A500095F690001507E03A9511900095E9900015B9",
                    INIT_04 => X"B10090010000204D108011961298204D1040110D1203204D1010112712005000",
                    INIT_05 => X"00571200B654500092046058D2019205D208B6645000B67400565000604DB200",
                    INIT_06 => X"005D580450000620007007200057005D2070005D500002700057028000570290",
                    INIT_07 => X"120000571200B654206E191F006E191E207002600057005D58045000B6740057",
                    INIT_08 => X"12FF005712FF0057120100571201005712A8005712C000571280005712040057",
                    INIT_09 => X"12030057120200571201005712DC005712080057120000571200005712FF0057",
                    INIT_0A => X"60ABC67016010057A2602129B6740057120200571201005712A8005712C00057",
                    INIT_0B => X"0057120000571200B654607CD280B20060B1D204006618001939203C15655000",
                    INIT_0C => X"190CB6540073B615B7141904006EB71318081900B67400AB1713160000571204",
                    INIT_0D => X"206E004117101901500061101000D1FFB11C0129B67400AB171C1616005D180C",
                    INIT_0E => X"182819031000D1FF700061107001206E0041170819010049B6740045B2740045",
                    INIT_0F => X"20DCD21B20DCD21A20DCD21820E5D21C182C215AD2172148D2002108D2140066",
                    INIT_10 => X"D2172325D2000066180819031000D100003C153F214FD21320DCD21E20DCD21D",
                    INIT_11 => X"20DCD21020DCD21E20DCD21D20DCD21B20DCD21A20DCD21820E0D21C180C2337",
                    INIT_12 => X"18481700007818280078180817085000003C15452128D2142335D215232CD213",
                    INIT_13 => X"190000DC182800DC1808007818E8007818C8007818A800781888007818680078",
                    INIT_14 => X"19012041006E1701182C1901003C154F20731650170019047100006E17011828",
                    INIT_15 => X"6FF06CC0700100681828192650007F007E007D007C007B007A00006E1702182C",
                    INIT_16 => X"0068182819280B700A601000F700D600219BF700D60073006168FFFFDC047000",
                    INIT_17 => X"DC04350014011301E2300057140015001300005712380057026000570270B654",
                    INIT_18 => X"192827900680B674095008406330617AE5B0C4A02189D3FF2189D20A2184FFFF",
                    INIT_19 => X"6990688067706330700150007000DFFF41A9DFFF7001006E174019010073182C",
                    INIT_1A => X"DF0061AAC4301401003CA54014006BA01000FB00DA0070006FF06CC06BB06AA0",
                    INIT_1B => X"61B060A050001FFE61BEF40AF50DD302B401B500500013001F0141B6DF0001FB",
                    INIT_1C => X"1B0321D4D2801415174003BB19071800170061DE02DD1AF61B02699068806770",
                    INIT_1D => X"6A007000222DE1D414011701622DC25021DED200A54003BB21DE622D02DE1A0B",
                    INIT_1E => X"90000030002A003CA5A01B0D1A07170050001E001D0070016990688067706B10",
                    INIT_1F => X"02DD1AE81B021C001D00500061EBCAB01A01475061F190014900480047061004",
                    INIT_20 => X"22071C026210D262A22D01E850001A5E1B02170018001906620AD220B2056222",
                    INIT_21 => X"1B021C03622DD27250001A5E1B021C05621CD27350001A5A1B021C036216D265",
                    INIT_22 => X"1A561B021C0050001A311B021C04A22D01E81C02622D02DD1AEE1B0250001A52",
                    INIT_23 => X"1E005000225E7000FE1FDDFF3E001D0170010041623FD3007000633070015000",
                    INIT_24 => X"1D0003F822402240DE0F224ECE301E01700039003800170103D570016240A4E0",
                    INIT_25 => X"6CC07001700003F803E67001225E170018FE19064BA01A531B02025E50001E00",
                    INIT_26 => X"02BA70001C00170018FF19066D706B806A907001006818281924227ADC007000",
                    INIT_27 => X"02BA2283DC050068182819242281DC032281DC041D006CC067D068B069A002CB",
                    INIT_28 => X"6295D902229400DC00E56294D100180C0180628ED9010290700120E5182C02CB",
                    INIT_29 => X"1903F20A006618081900F109F80003B01300F800588018007001036C70006380",
                    INIT_2A => X"026000570270B654700068202280700062A3D80C180102AAA2801800F20B0066",
                    INIT_2B => X"3800170103BB1440151F70015000700137001601B67400570280005712340057",
                    INIT_2C => X"172019010073182C19245000700062BDB500940102AA22C9D20022C5DC023900",
                    INIT_2D => X"D5004BA01400500062D1F720D6000068182819205000D217006618281903006E",
                    INIT_2E => X"154F15501500152F152015541545154722DE3B001A0114015000C560A6401000",
                    INIT_2F => X"15741561157A15691572156F15681574157515411500155F152F152015541553",
                    INIT_30 => X"156115741552155715591500156315691573156115421520153A156E156F1569",
                    INIT_31 => X"80263E001D0170011500153D1534155715611574155215571559153615341557",
                    INIT_32 => X"180C1901031C752B206E1701180C19016041031C752E50007000003CFE03DDFF",
                    INIT_33 => X"5000D20292059000F710D600006818081926231C752A2041006E700067107001",
                    INIT_34 => X"16007001005712180057026000570270B6546460657000681808192867706660",
                    INIT_35 => X"64407001B6746354B7009601D20292046355D2039205D2087000677066601710",
                    INIT_36 => X"120000571200B634206E174019010073180C1928275006407000677066606550",
                    INIT_37 => X"00571201B6340045036C130023ABB67400575202023000571200005712600057",
                    INIT_38 => X"12000057120000571203B6340045B674005712200057120C0057120000571200",
                    INIT_39 => X"005712410057121F0057120900571202B6340045B674005712FF0057127F0057",
                    INIT_3A => X"5000B6740045B6F4004503B0131D63ABF30ED280B31DB2000045B6740057122A",
                    INIT_3B => X"028000560290005612035000B67463B3C3401301E2300057A23014090430B634",
                    INIT_3C => X"D3FFE23000561300005602700056028000560290005612032053005602700056",
                    INIT_3D => X"02D00056027000560280005602900056120203F60D205000B67423CD130123D3",
                    INIT_3E => X"03F3005302700056028000560290005612D803F6500003F863E1D20103F30053",
                    INIT_3F => X"0000000000000000000000002053120420531206205300561205500063EFD201",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"22888A2A888AA8888A30AAAD52020202B099DDDD8DD88AAA8C00A3562288A6AA",
                   INITP_01 => X"DDDD37760DDE82AAA09D2A0828080A0888B4D822D62A22222222222222222222",
                   INITP_02 => X"55A022228035D5D75E095560280880602222222208828DDDDDDDDDD3760D8DDD",
                   INITP_03 => X"802D4D54EA0083557B5DD2B83420381558D508777582D75555FDDE0816075DDD",
                   INITP_04 => X"B60DD156B00578377EB808282B35D5B42F55EDDE02038E080D80D80D8DE800D3",
                   INITP_05 => X"AAAAAAAAAAAA95D362D60D82082F5B75583B5A2222DBD60A282A20EDDAB434E2",
                   INITP_06 => X"8A8A8222228205D57B58C2D438889605C35826B70980E6F1D7AAAAAAAAAAAAAA",
                   INITP_07 => X"00088A2CA2222ACA222222A7688888A2222B5A12AAA350A88888AA22222A8888",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a(31 downto 0),
                      DOPADOP => data_out_a(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b(31 downto 0),
                      DOPBDOP => data_out_b(35 downto 32), 
                        DIBDI => data_in_b(31 downto 0),
                      DIPBDIP => data_in_b(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_2k_generate;
  --
  --	
  ram_4k_generate : if (C_RAM_SIZE_KWORDS = 4) generate
    s6: if (C_FAMILY = "S6") generate
      --
      address_a(13 downto 0) <= address(10 downto 0) & "000";
      data_in_a <= "000000000000000000000000000000000000";
      --
      s6_a11_flop: FD
      port map (  D => address(11),
                  Q => pipe_a11,
                  C => clk);
      --
      s6_4k_mux0_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(0),
                I1 => data_out_a_hl(0),
                I2 => data_out_a_ll(1),
                I3 => data_out_a_hl(1),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(0),
                O6 => instruction(1));
      --
      s6_4k_mux2_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(2),
                I1 => data_out_a_hl(2),
                I2 => data_out_a_ll(3),
                I3 => data_out_a_hl(3),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(2),
                O6 => instruction(3));
      --
      s6_4k_mux4_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(4),
                I1 => data_out_a_hl(4),
                I2 => data_out_a_ll(5),
                I3 => data_out_a_hl(5),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(4),
                O6 => instruction(5));
      --
      s6_4k_mux6_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(6),
                I1 => data_out_a_hl(6),
                I2 => data_out_a_ll(7),
                I3 => data_out_a_hl(7),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(6),
                O6 => instruction(7));
      --
      s6_4k_mux8_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_ll(32),
                I1 => data_out_a_hl(32),
                I2 => data_out_a_lh(0),
                I3 => data_out_a_hh(0),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(8),
                O6 => instruction(9));
      --
      s6_4k_mux10_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(1),
                I1 => data_out_a_hh(1),
                I2 => data_out_a_lh(2),
                I3 => data_out_a_hh(2),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(10),
                O6 => instruction(11));
      --
      s6_4k_mux12_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(3),
                I1 => data_out_a_hh(3),
                I2 => data_out_a_lh(4),
                I3 => data_out_a_hh(4),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(12),
                O6 => instruction(13));
      --
      s6_4k_mux14_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(5),
                I1 => data_out_a_hh(5),
                I2 => data_out_a_lh(6),
                I3 => data_out_a_hh(6),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(14),
                O6 => instruction(15));
      --
      s6_4k_mux16_lut: LUT6_2
      generic map (INIT => X"FF00F0F0CCCCAAAA")
      port map( I0 => data_out_a_lh(7),
                I1 => data_out_a_hh(7),
                I2 => data_out_a_lh(32),
                I3 => data_out_a_hh(32),
                I4 => pipe_a11,
                I5 => '1',
                O5 => instruction(16),
                O6 => instruction(17));
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_ll <= "000" & data_out_b_ll(32) & "000000000000000000000000" & data_out_b_ll(7 downto 0);
        data_in_b_lh <= "000" & data_out_b_lh(32) & "000000000000000000000000" & data_out_b_lh(7 downto 0);
        data_in_b_hl <= "000" & data_out_b_hl(32) & "000000000000000000000000" & data_out_b_hl(7 downto 0);
        data_in_b_hh <= "000" & data_out_b_hh(32) & "000000000000000000000000" & data_out_b_hh(7 downto 0);
        address_b(13 downto 0) <= "00000000000000";
        we_b_l(3 downto 0) <= "0000";
        we_b_h(3 downto 0) <= "0000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
        jtag_dout <= data_out_b_lh(32) & data_out_b_lh(7 downto 0) & data_out_b_ll(32) & data_out_b_ll(7 downto 0);
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_lh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_ll <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        data_in_b_hh <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_hl <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b(13 downto 0) <= jtag_addr(10 downto 0) & "000";
        --
        s6_4k_jtag_we_lut: LUT6_2
        generic map (INIT => X"8000000020000000")
        port map( I0 => jtag_we,
                  I1 => jtag_addr(11),
                  I2 => '1',
                  I3 => '1',
                  I4 => '1',
                  I5 => '1',
                  O5 => jtag_we_l,
                  O6 => jtag_we_h);
        --
        we_b_l(3 downto 0) <= jtag_we_l & jtag_we_l & jtag_we_l & jtag_we_l;
        we_b_h(3 downto 0) <= jtag_we_h & jtag_we_h & jtag_we_h & jtag_we_h;
        --
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
        --
        s6_4k_jtag_mux0_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(0),
                  I1 => data_out_b_hl(0),
                  I2 => data_out_b_ll(1),
                  I3 => data_out_b_hl(1),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(0),
                  O6 => jtag_dout(1));
        --
        s6_4k_jtag_mux2_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(2),
                  I1 => data_out_b_hl(2),
                  I2 => data_out_b_ll(3),
                  I3 => data_out_b_hl(3),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(2),
                  O6 => jtag_dout(3));
        --
        s6_4k_jtag_mux4_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(4),
                  I1 => data_out_b_hl(4),
                  I2 => data_out_b_ll(5),
                  I3 => data_out_b_hl(5),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(4),
                  O6 => jtag_dout(5));
        --
        s6_4k_jtag_mux6_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(6),
                  I1 => data_out_b_hl(6),
                  I2 => data_out_b_ll(7),
                  I3 => data_out_b_hl(7),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(6),
                  O6 => jtag_dout(7));
        --
        s6_4k_jtag_mux8_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_ll(32),
                  I1 => data_out_b_hl(32),
                  I2 => data_out_b_lh(0),
                  I3 => data_out_b_hh(0),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(8),
                  O6 => jtag_dout(9));
        --
        s6_4k_jtag_mux10_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(1),
                  I1 => data_out_b_hh(1),
                  I2 => data_out_b_lh(2),
                  I3 => data_out_b_hh(2),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(10),
                  O6 => jtag_dout(11));
        --
        s6_4k_jtag_mux12_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(3),
                  I1 => data_out_b_hh(3),
                  I2 => data_out_b_lh(4),
                  I3 => data_out_b_hh(4),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(12),
                  O6 => jtag_dout(13));
        --
        s6_4k_jtag_mux14_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(5),
                  I1 => data_out_b_hh(5),
                  I2 => data_out_b_lh(6),
                  I3 => data_out_b_hh(6),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(14),
                  O6 => jtag_dout(15));
        --
        s6_4k_jtag_mux16_lut: LUT6_2
        generic map (INIT => X"FF00F0F0CCCCAAAA")
        port map( I0 => data_out_b_lh(7),
                  I1 => data_out_b_hh(7),
                  I2 => data_out_b_lh(32),
                  I3 => data_out_b_hh(32),
                  I4 => jtag_addr(11),
                  I5 => '1',
                  O5 => jtag_dout(16),
                  O6 => jtag_dout(17));
      --
      end generate loader;
      --
      kcpsm6_rom_ll: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"3C52214000000007173C66181520205320562056569F3C0C0131FF4974457449",
                    INIT_01 => X"013C0400000A00F600073A1100E900B900DF007B00613C0A3C0D24E97AB3C4E6",
                    INIT_02 => X"57005400045801050864007456004D000001004D8096984D400D034D10270000",
                    INIT_03 => X"005700546E1F6E1E7060575D040074575D0400207020575D705D007057805790",
                    INIT_04 => X"035702570157DC57085700570057FF57FF57FF5701570157A857C05780570457",
                    INIT_05 => X"57005700547C8000B1046600393C6500AB7001576029745702570157A857C057",
                    INIT_06 => X"6E411001001000FF1C2974AB1C165D0C0C54731514046E13080074AB13005704",
                    INIT_07 => X"DC1BDC1ADC18E51C2C5A174800081466280300FF0010016E4108014974457445",
                    INIT_08 => X"DC10DC1EDC1DDC1BDC1ADC18E01C0C3717250066080300003C3F4F13DC1EDC1D",
                    INIT_09 => X"00DC28DC0878E878C878A8788878687848007828780808003C45281435152C13",
                    INIT_0A => X"F0C001682826000000000000006E022C01416E012C013C4F73500004006E0128",
                    INIT_0B => X"0400010130570000005738576057705468282870600000009B00000068FF0400",
                    INIT_0C => X"90807030010000FFA9FF016E4001732C289080745040307AB0A089FF890A84FF",
                    INIT_0D => X"B0A000FEBE0A0D020100000001B600FB00AA30013C4000A000000000F0C0B0A0",
                    INIT_0E => X"00002DD401012D50DE0040BBDE2DDE0B03D4801540BB070000DEDDF602908070",
                    INIT_0F => X"DDE802000000EBB00150F1010000060400302A3CA00D07000000000190807010",
                    INIT_10 => X"02032D72005E02051C73005A02031665070210622DE8005E020000060A200522",
                    INIT_11 => X"00005E001FFF000101413F0000300100560200003102042DE8022DDDEE020052",
                    INIT_12 => X"C00100F8E6015E00FE06A053025E000000F840400F4E300100000001D50140E0",
                    INIT_13 => X"BA83056828248103810400C0D0B0A0CBBA000000FF06708090016828247A0000",
                    INIT_14 => X"030A6608000900B000008000016C0080950294DCE594000C808E019001E52CCB",
                    INIT_15 => X"0001BB401F01000100017457805734576057705400208000A30C01AA80000B66",
                    INIT_16 => X"00A00000D1200068282000176628036E2001732C240000BD0001AAC900C50200",
                    INIT_17 => X"74617A69726F68747541005F2F2054534F50002F20544547DE00010100604000",
                    INIT_18 => X"26000101003D345761745257593634576174525759006369736142203A6E6F69",
                    INIT_19 => X"0002050010006808261C2A416E0010010C011C2B6E010C01411C2E00003C03FF",
                    INIT_1A => X"4001745400010204550305080070601000015718576057705460706808287060",
                    INIT_1B => X"570134456C00AB745702305700576057005700346E4001730C28504000706050",
                    INIT_1C => X"5741571F5709570234457457FF577F5700570057033445745720570C57005700",
                    INIT_1D => X"80569056030074B34001305730093034007445F445B01DAB0E801D004574572A",
                    INIT_1E => X"D05670568056905602F6200074CD01D3FF305600567056805690560353567056",
                    INIT_1F => X"0000000000005304530653560500EF01F353705680569056D8F600F8E101F353",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0054546035C88D48000A04000000000005000400000084448555554B45222122",
                   INITP_01 => X"2A6A0D2B14B8CECD9B5FC54AB1CCCBBB518032DC8552953A8000426A00014560",
                   INITP_02 => X"FFFFFF44C442C881688000008D8542A0042A17081968C05805144484820804D0",
                   INITP_03 => X"0002805A0067900001C007A0000000000E0006651885002608E275612FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_ll(31 downto 0),
                  DOPA => data_out_a_ll(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_ll(31 downto 0),
                  DOPB => data_out_b_ll(35 downto 32), 
                   DIB => data_in_b_ll(31 downto 0),
                  DIPB => data_in_b_ll(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_lh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"000AB068480B0C0C10000A90F9FCEC00040004000009000A5858B8005B005800",
                    INIT_01 => X"6AB06848288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100A000A100001000101",
                    INIT_02 => X"00095B2849B06949695B285B0028B0D9D8C80010080809100808091008080928",
                    INIT_03 => X"0900095B100C000C100100002C285B00002C2803000300001000280100010001",
                    INIT_04 => X"0900090009000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"000900095BB0E959B0E9000C0C100A28B0E38B0051105B000900090009000900",
                    INIT_06 => X"10000B0C28B088E858005B000B0B000C0C5B005B5B0C005B0C0C5B000B0B0009",
                    INIT_07 => X"90E990E990E990E90C90E990E990E9000C0C88E8B8B0B810000B0C005B005900",
                    INIT_08 => X"90E990E990E990E990E990E990E90C91E991E9000C0C88E8000A90E990E990E9",
                    INIT_09 => X"0C000C000C000C000C000C000C000C000C0B000C000C0B28000A90E991E991E9",
                    INIT_0A => X"B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C000A100B0B0CB8000B0C",
                    INIT_0B => X"EE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8",
                    INIT_0C => X"B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FF",
                    INIT_0D => X"B0B0280FB0FAFAE95A5A28098FA0EF80EFB0E28A00520A3588FDEDB8B7B6B5B5",
                    INIT_0E => X"B5B811F08A8BB1E190E9520110B1010D0D90E90A0B010C0C0BB0010D0DB4B4B3",
                    INIT_0F => X"010D0D0E0E28B0E58D23B0C8A4A4A308C8000000520D0D0B280F0EB8B4B4B3B5",
                    INIT_10 => X"0D0EB1E9280D0D0EB1E9280D0D0EB1E9110EB1E9D100280D0D0B0C0CB1E959B1",
                    INIT_11 => X"0F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0E280D0D0ED1000EB1010D0D280D",
                    INIT_12 => X"B6B8B80101B8110B0C0C250D0D01280F0E0111916F91E78FB89C9C8B01B8B152",
                    INIT_13 => X"0191EE000C0C91EE91EE0EB6B3B4B40101B80E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_14 => X"0C79000C0C787C01097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_15 => X"9C8B010A0AB828B89B8B5B00010009000100015BB8B411B8B1EC8C01510C7900",
                    INIT_16 => X"EA250A28B1FBEB000C0CA8E9000C0C000B0C000C0C28B8B1DACA0191E991EE9C",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388",
                    INIT_18 => X"C09F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_19 => X"A86949C8FBEB000C0C11BA1000B8B3B80C0C01BA100B0C0CB001BA28B8807FEE",
                    INIT_1A => X"B2B85BB1DBCB6949B1694969B8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1B => X"00095B000109115B00290100090009000900095B100B0C000C0C9383B8B3B3B2",
                    INIT_1C => X"00090009000900095B005B000900090009000900095B005B0009000900090009",
                    INIT_1D => X"0100010009285BB1E1897100518A025B285B005B000109B1F9E95959005B0009",
                    INIT_1E => X"0100010001000100090106285B118991E9710009000100010001000910000100",
                    INIT_1F => X"0000000000001009100910000928B169010001000100010009012801B1690100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"AA552B9FCA726232ACA59755555555555AB7ABEAB4FE1111CAAAAABFA0D15ADF",
                   INITP_01 => X"8620F090729E4460282549900EB2112A0C55848932046284555529AAAAA952AA",
                   INITP_02 => X"FFFFFF895929273427355B93674EBC4DD281C0657E26748C70EB11B228A2AE09",
                   INITP_03 => X"02B6D57B555D6AAD5731FD0EAAF557AABB95590872986A90925D28DC9FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_lh(31 downto 0),
                  DOPA => data_out_a_lh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_lh(31 downto 0),
                  DOPB => data_out_b_lh(35 downto 32), 
                   DIB => data_in_b_lh(31 downto 0),
                  DIPB => data_in_b_lh(35 downto 32), 
                   WEB => we_b_l(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      --
      kcpsm6_rom_hl: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hl(31 downto 0),
                  DOPA => data_out_a_hl(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hl(31 downto 0),
                  DOPB => data_out_b_hl(35 downto 32), 
                   DIB => data_in_b_hl(31 downto 0),
                  DIPB => data_in_b_hl(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
      -- 
      kcpsm6_rom_hh: RAMB16BWER
      generic map ( DATA_WIDTH_A => 9,
                    DOA_REG => 0,
                    EN_RSTRAM_A => FALSE,
                    INIT_A => X"000000000",
                    RST_PRIORITY_A => "CE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    DATA_WIDTH_B => 9,
                    DOB_REG => 0,
                    EN_RSTRAM_B => FALSE,
                    INIT_B => X"000000000",
                    RST_PRIORITY_B => "CE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    RSTTYPE => "SYNC",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    SIM_DEVICE => "SPARTAN6",
                    INIT_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_0F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_10 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_11 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_12 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_13 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_14 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_15 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_16 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_17 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_18 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_19 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_1F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_01 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_02 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_03 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(  ADDRA => address_a(13 downto 0),
                   ENA => enable,
                  CLKA => clk,
                   DOA => data_out_a_hh(31 downto 0),
                  DOPA => data_out_a_hh(35 downto 32), 
                   DIA => data_in_a(31 downto 0),
                  DIPA => data_in_a(35 downto 32), 
                   WEA => "0000",
                REGCEA => '0',
                  RSTA => '0',
                 ADDRB => address_b(13 downto 0),
                   ENB => enable_b,
                  CLKB => clk_b,
                   DOB => data_out_b_hh(31 downto 0),
                  DOPB => data_out_b_hh(35 downto 32), 
                   DIB => data_in_b_hh(31 downto 0),
                  DIPB => data_in_b_hh(35 downto 32), 
                   WEB => we_b_h(3 downto 0),
                REGCEB => '0',
                  RSTB => '0');
    --
    end generate s6;
    --
    --
    v6 : if (C_FAMILY = "V6") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"3C52214000000007173C66181520205320562056569F3C0C0131FF4974457449",
                    INIT_01 => X"013C0400000A00F600073A1100E900B900DF007B00613C0A3C0D24E97AB3C4E6",
                    INIT_02 => X"57005400045801050864007456004D000001004D8096984D400D034D10270000",
                    INIT_03 => X"005700546E1F6E1E7060575D040074575D0400207020575D705D007057805790",
                    INIT_04 => X"035702570157DC57085700570057FF57FF57FF5701570157A857C05780570457",
                    INIT_05 => X"57005700547C8000B1046600393C6500AB7001576029745702570157A857C057",
                    INIT_06 => X"6E411001001000FF1C2974AB1C165D0C0C54731514046E13080074AB13005704",
                    INIT_07 => X"DC1BDC1ADC18E51C2C5A174800081466280300FF0010016E4108014974457445",
                    INIT_08 => X"DC10DC1EDC1DDC1BDC1ADC18E01C0C3717250066080300003C3F4F13DC1EDC1D",
                    INIT_09 => X"00DC28DC0878E878C878A8788878687848007828780808003C45281435152C13",
                    INIT_0A => X"F0C001682826000000000000006E022C01416E012C013C4F73500004006E0128",
                    INIT_0B => X"0400010130570000005738576057705468282870600000009B00000068FF0400",
                    INIT_0C => X"90807030010000FFA9FF016E4001732C289080745040307AB0A089FF890A84FF",
                    INIT_0D => X"B0A000FEBE0A0D020100000001B600FB00AA30013C4000A000000000F0C0B0A0",
                    INIT_0E => X"00002DD401012D50DE0040BBDE2DDE0B03D4801540BB070000DEDDF602908070",
                    INIT_0F => X"DDE802000000EBB00150F1010000060400302A3CA00D07000000000190807010",
                    INIT_10 => X"02032D72005E02051C73005A02031665070210622DE8005E020000060A200522",
                    INIT_11 => X"00005E001FFF000101413F0000300100560200003102042DE8022DDDEE020052",
                    INIT_12 => X"C00100F8E6015E00FE06A053025E000000F840400F4E300100000001D50140E0",
                    INIT_13 => X"BA83056828248103810400C0D0B0A0CBBA000000FF06708090016828247A0000",
                    INIT_14 => X"030A6608000900B000008000016C0080950294DCE594000C808E019001E52CCB",
                    INIT_15 => X"0001BB401F01000100017457805734576057705400208000A30C01AA80000B66",
                    INIT_16 => X"00A00000D1200068282000176628036E2001732C240000BD0001AAC900C50200",
                    INIT_17 => X"74617A69726F68747541005F2F2054534F50002F20544547DE00010100604000",
                    INIT_18 => X"26000101003D345761745257593634576174525759006369736142203A6E6F69",
                    INIT_19 => X"0002050010006808261C2A416E0010010C011C2B6E010C01411C2E00003C03FF",
                    INIT_1A => X"4001745400010204550305080070601000015718576057705460706808287060",
                    INIT_1B => X"570134456C00AB745702305700576057005700346E4001730C28504000706050",
                    INIT_1C => X"5741571F5709570234457457FF577F5700570057033445745720570C57005700",
                    INIT_1D => X"80569056030074B34001305730093034007445F445B01DAB0E801D004574572A",
                    INIT_1E => X"D05670568056905602F6200074CD01D3FF305600567056805690560353567056",
                    INIT_1F => X"0000000000005304530653560500EF01F353705680569056D8F600F8E101F353",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0054546035C88D48000A04000000000005000400000084448555554B45222122",
                   INITP_01 => X"2A6A0D2B14B8CECD9B5FC54AB1CCCBBB518032DC8552953A8000426A00014560",
                   INITP_02 => X"FFFFFF44C442C881688000008D8542A0042A17081968C05805144484820804D0",
                   INITP_03 => X"0002805A0067900001C007A0000000000E0006651885002608E275612FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "VIRTEX6",
                    INIT_00 => X"000AB068480B0C0C10000A90F9FCEC00040004000009000A5858B8005B005800",
                    INIT_01 => X"6AB06848288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100A000A100001000101",
                    INIT_02 => X"00095B2849B06949695B285B0028B0D9D8C80010080809100808091008080928",
                    INIT_03 => X"0900095B100C000C100100002C285B00002C2803000300001000280100010001",
                    INIT_04 => X"0900090009000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"000900095BB0E959B0E9000C0C100A28B0E38B0051105B000900090009000900",
                    INIT_06 => X"10000B0C28B088E858005B000B0B000C0C5B005B5B0C005B0C0C5B000B0B0009",
                    INIT_07 => X"90E990E990E990E90C90E990E990E9000C0C88E8B8B0B810000B0C005B005900",
                    INIT_08 => X"90E990E990E990E990E990E990E90C91E991E9000C0C88E8000A90E990E990E9",
                    INIT_09 => X"0C000C000C000C000C000C000C000C000C0B000C000C0B28000A90E991E991E9",
                    INIT_0A => X"B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C000A100B0B0CB8000B0C",
                    INIT_0B => X"EE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8",
                    INIT_0C => X"B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FF",
                    INIT_0D => X"B0B0280FB0FAFAE95A5A28098FA0EF80EFB0E28A00520A3588FDEDB8B7B6B5B5",
                    INIT_0E => X"B5B811F08A8BB1E190E9520110B1010D0D90E90A0B010C0C0BB0010D0DB4B4B3",
                    INIT_0F => X"010D0D0E0E28B0E58D23B0C8A4A4A308C8000000520D0D0B280F0EB8B4B4B3B5",
                    INIT_10 => X"0D0EB1E9280D0D0EB1E9280D0D0EB1E9110EB1E9D100280D0D0B0C0CB1E959B1",
                    INIT_11 => X"0F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0E280D0D0ED1000EB1010D0D280D",
                    INIT_12 => X"B6B8B80101B8110B0C0C250D0D01280F0E0111916F91E78FB89C9C8B01B8B152",
                    INIT_13 => X"0191EE000C0C91EE91EE0EB6B3B4B40101B80E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_14 => X"0C79000C0C787C01097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_15 => X"9C8B010A0AB828B89B8B5B00010009000100015BB8B411B8B1EC8C01510C7900",
                    INIT_16 => X"EA250A28B1FBEB000C0CA8E9000C0C000B0C000C0C28B8B1DACA0191E991EE9C",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388",
                    INIT_18 => X"C09F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_19 => X"A86949C8FBEB000C0C11BA1000B8B3B80C0C01BA100B0C0CB001BA28B8807FEE",
                    INIT_1A => X"B2B85BB1DBCB6949B1694969B8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1B => X"00095B000109115B00290100090009000900095B100B0C000C0C9383B8B3B3B2",
                    INIT_1C => X"00090009000900095B005B000900090009000900095B005B0009000900090009",
                    INIT_1D => X"0100010009285BB1E1897100518A025B285B005B000109B1F9E95959005B0009",
                    INIT_1E => X"0100010001000100090106285B118991E9710009000100010001000910000100",
                    INIT_1F => X"0000000000001009100910000928B169010001000100010009012801B1690100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"AA552B9FCA726232ACA59755555555555AB7ABEAB4FE1111CAAAAABFA0D15ADF",
                   INITP_01 => X"8620F090729E4460282549900EB2112A0C55848932046284555529AAAAA952AA",
                   INITP_02 => X"FFFFFF895929273427355B93674EBC4DD281C0657E26748C70EB11B228A2AE09",
                   INITP_03 => X"02B6D57B555D6AAD5731FD0EAAF557AABB95590872986A90925D28DC9FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate v6;
    --
    --
    akv7 : if (C_FAMILY = "7S") generate
      --
      address_a <= '1' & address(11 downto 0) & "111";
      instruction <= data_out_a_h(32) & data_out_a_h(7 downto 0) & data_out_a_l(32) & data_out_a_l(7 downto 0);
      data_in_a <= "000000000000000000000000000000000000";
      jtag_dout <= data_out_b_h(32) & data_out_b_h(7 downto 0) & data_out_b_l(32) & data_out_b_l(7 downto 0);
      --
      no_loader : if (C_JTAG_LOADER_ENABLE = 0) generate
        data_in_b_l <= "000" & data_out_b_l(32) & "000000000000000000000000" & data_out_b_l(7 downto 0);
        data_in_b_h <= "000" & data_out_b_h(32) & "000000000000000000000000" & data_out_b_h(7 downto 0);
        address_b <= "1111111111111111";
        we_b <= "00000000";
        enable_b <= '0';
        rdl <= '0';
        clk_b <= '0';
      end generate no_loader;
      --
      loader : if (C_JTAG_LOADER_ENABLE = 1) generate
        data_in_b_h <= "000" & jtag_din(17) & "000000000000000000000000" & jtag_din(16 downto 9);
        data_in_b_l <= "000" & jtag_din(8) & "000000000000000000000000" & jtag_din(7 downto 0);
        address_b <= '1' & jtag_addr(11 downto 0) & "111";
        we_b <= jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we & jtag_we;
        enable_b <= jtag_en(0);
        rdl <= rdl_bus(0);
        clk_b <= jtag_clk;
      end generate loader;
      --
      kcpsm6_rom_l: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"3C52214000000007173C66181520205320562056569F3C0C0131FF4974457449",
                    INIT_01 => X"013C0400000A00F600073A1100E900B900DF007B00613C0A3C0D24E97AB3C4E6",
                    INIT_02 => X"57005400045801050864007456004D000001004D8096984D400D034D10270000",
                    INIT_03 => X"005700546E1F6E1E7060575D040074575D0400207020575D705D007057805790",
                    INIT_04 => X"035702570157DC57085700570057FF57FF57FF5701570157A857C05780570457",
                    INIT_05 => X"57005700547C8000B1046600393C6500AB7001576029745702570157A857C057",
                    INIT_06 => X"6E411001001000FF1C2974AB1C165D0C0C54731514046E13080074AB13005704",
                    INIT_07 => X"DC1BDC1ADC18E51C2C5A174800081466280300FF0010016E4108014974457445",
                    INIT_08 => X"DC10DC1EDC1DDC1BDC1ADC18E01C0C3717250066080300003C3F4F13DC1EDC1D",
                    INIT_09 => X"00DC28DC0878E878C878A8788878687848007828780808003C45281435152C13",
                    INIT_0A => X"F0C001682826000000000000006E022C01416E012C013C4F73500004006E0128",
                    INIT_0B => X"0400010130570000005738576057705468282870600000009B00000068FF0400",
                    INIT_0C => X"90807030010000FFA9FF016E4001732C289080745040307AB0A089FF890A84FF",
                    INIT_0D => X"B0A000FEBE0A0D020100000001B600FB00AA30013C4000A000000000F0C0B0A0",
                    INIT_0E => X"00002DD401012D50DE0040BBDE2DDE0B03D4801540BB070000DEDDF602908070",
                    INIT_0F => X"DDE802000000EBB00150F1010000060400302A3CA00D07000000000190807010",
                    INIT_10 => X"02032D72005E02051C73005A02031665070210622DE8005E020000060A200522",
                    INIT_11 => X"00005E001FFF000101413F0000300100560200003102042DE8022DDDEE020052",
                    INIT_12 => X"C00100F8E6015E00FE06A053025E000000F840400F4E300100000001D50140E0",
                    INIT_13 => X"BA83056828248103810400C0D0B0A0CBBA000000FF06708090016828247A0000",
                    INIT_14 => X"030A6608000900B000008000016C0080950294DCE594000C808E019001E52CCB",
                    INIT_15 => X"0001BB401F01000100017457805734576057705400208000A30C01AA80000B66",
                    INIT_16 => X"00A00000D1200068282000176628036E2001732C240000BD0001AAC900C50200",
                    INIT_17 => X"74617A69726F68747541005F2F2054534F50002F20544547DE00010100604000",
                    INIT_18 => X"26000101003D345761745257593634576174525759006369736142203A6E6F69",
                    INIT_19 => X"0002050010006808261C2A416E0010010C011C2B6E010C01411C2E00003C03FF",
                    INIT_1A => X"4001745400010204550305080070601000015718576057705460706808287060",
                    INIT_1B => X"570134456C00AB745702305700576057005700346E4001730C28504000706050",
                    INIT_1C => X"5741571F5709570234457457FF577F5700570057033445745720570C57005700",
                    INIT_1D => X"80569056030074B34001305730093034007445F445B01DAB0E801D004574572A",
                    INIT_1E => X"D05670568056905602F6200074CD01D3FF305600567056805690560353567056",
                    INIT_1F => X"0000000000005304530653560500EF01F353705680569056D8F600F8E101F353",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"0054546035C88D48000A04000000000005000400000084448555554B45222122",
                   INITP_01 => X"2A6A0D2B14B8CECD9B5FC54AB1CCCBBB518032DC8552953A8000426A00014560",
                   INITP_02 => X"FFFFFF44C442C881688000008D8542A0042A17081968C05805144484820804D0",
                   INITP_03 => X"0002805A0067900001C007A0000000000E0006651885002608E275612FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_l(31 downto 0),
                      DOPADOP => data_out_a_l(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_l(31 downto 0),
                      DOPBDOP => data_out_b_l(35 downto 32), 
                        DIBDI => data_in_b_l(31 downto 0),
                      DIPBDIP => data_in_b_l(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
      kcpsm6_rom_h: RAMB36E1
      generic map ( READ_WIDTH_A => 9,
                    WRITE_WIDTH_A => 9,
                    DOA_REG => 0,
                    INIT_A => X"000000000",
                    RSTREG_PRIORITY_A => "REGCE",
                    SRVAL_A => X"000000000",
                    WRITE_MODE_A => "WRITE_FIRST",
                    READ_WIDTH_B => 9,
                    WRITE_WIDTH_B => 9,
                    DOB_REG => 0,
                    INIT_B => X"000000000",
                    RSTREG_PRIORITY_B => "REGCE",
                    SRVAL_B => X"000000000",
                    WRITE_MODE_B => "WRITE_FIRST",
                    INIT_FILE => "NONE",
                    SIM_COLLISION_CHECK => "ALL",
                    RAM_MODE => "TDP",
                    RDADDR_COLLISION_HWCONFIG => "DELAYED_WRITE",
                    EN_ECC_READ => FALSE,
                    EN_ECC_WRITE => FALSE,
                    RAM_EXTENSION_A => "NONE",
                    RAM_EXTENSION_B => "NONE",
                    SIM_DEVICE => "7SERIES",
                    INIT_00 => X"000AB068480B0C0C10000A90F9FCEC00040004000009000A5858B8005B005800",
                    INIT_01 => X"6AB06848288A28CAC88AF0CAC8CAC88A281AE8EAC8EA100A000A100001000101",
                    INIT_02 => X"00095B2849B06949695B285B0028B0D9D8C80010080809100808091008080928",
                    INIT_03 => X"0900095B100C000C100100002C285B00002C2803000300001000280100010001",
                    INIT_04 => X"0900090009000900090009000900090009000900090009000900090009000900",
                    INIT_05 => X"000900095BB0E959B0E9000C0C100A28B0E38B0051105B000900090009000900",
                    INIT_06 => X"10000B0C28B088E858005B000B0B000C0C5B005B5B0C005B0C0C5B000B0B0009",
                    INIT_07 => X"90E990E990E990E90C90E990E990E9000C0C88E8B8B0B810000B0C005B005900",
                    INIT_08 => X"90E990E990E990E990E990E990E90C91E991E9000C0C88E8000A90E990E990E9",
                    INIT_09 => X"0C000C000C000C000C000C000C000C000C0B000C000C0B28000A90E991E991E9",
                    INIT_0A => X"B7B6B8000C0C28BFBFBEBEBDBD000B0C0C10000B0C0C000A100B0B0CB8000B0C",
                    INIT_0B => X"EE9A8A8971000A0A090009000100015B000C0C050588FBEB90FBEBB9B0FFEEB8",
                    INIT_0C => X"B4B4B3B1B8A8B8EFA0EFB8000B0C000C0C93835B0404B1B0F2E290E990E990FF",
                    INIT_0D => X"B0B0280FB0FAFAE95A5A28098FA0EF80EFB0E28A00520A3588FDEDB8B7B6B5B5",
                    INIT_0E => X"B5B811F08A8BB1E190E9520110B1010D0D90E90A0B010C0C0BB0010D0DB4B4B3",
                    INIT_0F => X"010D0D0E0E28B0E58D23B0C8A4A4A308C8000000520D0D0B280F0EB8B4B4B3B5",
                    INIT_10 => X"0D0EB1E9280D0D0EB1E9280D0D0EB1E9110EB1E9D100280D0D0B0C0CB1E959B1",
                    INIT_11 => X"0F2891B8FFEE9F8EB800B1E9B8B1B8280D0D0E280D0D0ED1000EB1010D0D280D",
                    INIT_12 => X"B6B8B80101B8110B0C0C250D0D01280F0E0111916F91E78FB89C9C8B01B8B152",
                    INIT_13 => X"0191EE000C0C91EE91EE0EB6B3B4B40101B80E0B0C0CB6B5B5B8000C0C91EEB8",
                    INIT_14 => X"0C79000C0C787C01097C2C0CB801B8B1B1EC110000B1E80C00B1EC01B8100C01",
                    INIT_15 => X"9C8B010A0AB828B89B8B5B00010009000100015BB8B411B8B1EC8C01510C7900",
                    INIT_16 => X"EA250A28B1FBEB000C0CA8E9000C0C000B0C000C0C28B8B1DACA0191E991EE9C",
                    INIT_17 => X"0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A119D8D8AA8E25388",
                    INIT_18 => X"C09F8EB80A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0A",
                    INIT_19 => X"A86949C8FBEB000C0C11BA1000B8B3B80C0C01BA100B0C0CB001BA28B8807FEE",
                    INIT_1A => X"B2B85BB1DBCB6949B1694969B8B3B30B0BB80009000100015BB2B2000C0CB3B3",
                    INIT_1B => X"00095B000109115B00290100090009000900095B100B0C000C0C9383B8B3B3B2",
                    INIT_1C => X"00090009000900095B005B000900090009000900095B005B0009000900090009",
                    INIT_1D => X"0100010009285BB1E1897100518A025B285B005B000109B1F9E95959005B0009",
                    INIT_1E => X"0100010001000100090106285B118991E9710009000100010001000910000100",
                    INIT_1F => X"0000000000001009100910000928B169010001000100010009012801B1690100",
                    INIT_20 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_21 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_22 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_23 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_24 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_25 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_26 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_27 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_28 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_29 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_2F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_30 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_31 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_32 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_33 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_34 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_35 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_36 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_37 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_38 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_39 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_3F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_40 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_41 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_42 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_43 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_44 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_45 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_46 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_47 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_48 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_49 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_4F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_50 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_51 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_52 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_53 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_54 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_55 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_56 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_57 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_58 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_59 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_5F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_60 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_61 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_62 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_63 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_64 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_65 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_66 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_67 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_68 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_69 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_6F => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_70 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_71 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_72 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_73 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_74 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_75 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_76 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_77 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_78 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_79 => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7A => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7B => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7C => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7D => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7E => X"0000000000000000000000000000000000000000000000000000000000000000",
                    INIT_7F => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_00 => X"AA552B9FCA726232ACA59755555555555AB7ABEAB4FE1111CAAAAABFA0D15ADF",
                   INITP_01 => X"8620F090729E4460282549900EB2112A0C55848932046284555529AAAAA952AA",
                   INITP_02 => X"FFFFFF895929273427355B93674EBC4DD281C0657E26748C70EB11B228A2AE09",
                   INITP_03 => X"02B6D57B555D6AAD5731FD0EAAF557AABB95590872986A90925D28DC9FFFFFFF",
                   INITP_04 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_05 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_06 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_07 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_08 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_09 => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0A => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0B => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0C => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0D => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0E => X"0000000000000000000000000000000000000000000000000000000000000000",
                   INITP_0F => X"0000000000000000000000000000000000000000000000000000000000000000")
      port map(   ADDRARDADDR => address_a,
                      ENARDEN => enable,
                    CLKARDCLK => clk,
                        DOADO => data_out_a_h(31 downto 0),
                      DOPADOP => data_out_a_h(35 downto 32), 
                        DIADI => data_in_a(31 downto 0),
                      DIPADIP => data_in_a(35 downto 32), 
                          WEA => "0000",
                  REGCEAREGCE => '0',
                RSTRAMARSTRAM => '0',
                RSTREGARSTREG => '0',
                  ADDRBWRADDR => address_b,
                      ENBWREN => enable_b,
                    CLKBWRCLK => clk_b,
                        DOBDO => data_out_b_h(31 downto 0),
                      DOPBDOP => data_out_b_h(35 downto 32), 
                        DIBDI => data_in_b_h(31 downto 0),
                      DIPBDIP => data_in_b_h(35 downto 32), 
                        WEBWE => we_b,
                       REGCEB => '0',
                      RSTRAMB => '0',
                      RSTREGB => '0',
                   CASCADEINA => '0',
                   CASCADEINB => '0',
                INJECTDBITERR => '0',
                INJECTSBITERR => '0');
      --
    end generate akv7;
    --
  end generate ram_4k_generate;	              
  --
  --
  --
  --
  -- JTAG Loader
  --
  instantiate_loader : if (C_JTAG_LOADER_ENABLE = 1) generate
  --
    jtag_loader_6_inst : jtag_loader_6
    generic map(              C_FAMILY => C_FAMILY,
                       C_NUM_PICOBLAZE => 1,
                  C_JTAG_LOADER_ENABLE => C_JTAG_LOADER_ENABLE,
                 C_BRAM_MAX_ADDR_WIDTH => BRAM_ADDRESS_WIDTH,
	                  C_ADDR_WIDTH_0 => BRAM_ADDRESS_WIDTH)
    port map( picoblaze_reset => rdl_bus,
                      jtag_en => jtag_en,
                     jtag_din => jtag_din,
                    jtag_addr => jtag_addr(BRAM_ADDRESS_WIDTH-1 downto 0),
                     jtag_clk => jtag_clk,
                      jtag_we => jtag_we,
                  jtag_dout_0 => jtag_dout,
                  jtag_dout_1 => jtag_dout, -- ports 1-7 are not used
                  jtag_dout_2 => jtag_dout, -- in a 1 device debug 
                  jtag_dout_3 => jtag_dout, -- session.  However, Synplify
                  jtag_dout_4 => jtag_dout, -- etc require all ports to
                  jtag_dout_5 => jtag_dout, -- be connected
                  jtag_dout_6 => jtag_dout,
                  jtag_dout_7 => jtag_dout);
    --  
  end generate instantiate_loader;
  --
end low_level_definition;
--
--
-------------------------------------------------------------------------------------------
--
-- JTAG Loader 
--
-------------------------------------------------------------------------------------------
--
--
-- JTAG Loader 6 - Version 6.00
-- Kris Chaplin 4 February 2010
-- Ken Chapman 15 August 2011 - Revised coding style
--
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
--
library unisim;
use unisim.vcomponents.all;
--
entity jtag_loader_6 is
generic(              C_JTAG_LOADER_ENABLE : integer := 1;
                                  C_FAMILY : string := "V6";
                           C_NUM_PICOBLAZE : integer := 1;
                     C_BRAM_MAX_ADDR_WIDTH : integer := 10;
        C_PICOBLAZE_INSTRUCTION_DATA_WIDTH : integer := 18;
                              C_JTAG_CHAIN : integer := 2;
                            C_ADDR_WIDTH_0 : integer := 10;
                            C_ADDR_WIDTH_1 : integer := 10;
                            C_ADDR_WIDTH_2 : integer := 10;
                            C_ADDR_WIDTH_3 : integer := 10;
                            C_ADDR_WIDTH_4 : integer := 10;
                            C_ADDR_WIDTH_5 : integer := 10;
                            C_ADDR_WIDTH_6 : integer := 10;
                            C_ADDR_WIDTH_7 : integer := 10);
port(   picoblaze_reset : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
                jtag_en : out std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
               jtag_din : out std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
              jtag_addr : out std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0) := (others => '0');
               jtag_clk : out std_logic := '0';
                jtag_we : out std_logic := '0';
            jtag_dout_0 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_1 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_2 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_3 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_4 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_5 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_6 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
            jtag_dout_7 : in  std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0));
end jtag_loader_6;
--
architecture Behavioral of jtag_loader_6 is
  --
  signal num_picoblaze       : std_logic_vector(2 downto 0);
  signal picoblaze_instruction_data_width : std_logic_vector(4 downto 0);
  --
  signal drck                : std_logic;
  signal shift_clk           : std_logic;
  signal shift_din           : std_logic;
  signal shift_dout          : std_logic;
  signal shift               : std_logic;
  signal capture             : std_logic;
  --
  signal control_reg_ce      : std_logic;
  signal bram_ce             : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal bus_zero            : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  signal jtag_en_int         : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0);
  signal jtag_en_expanded    : std_logic_vector(7 downto 0) := (others => '0');
  signal jtag_addr_int       : std_logic_vector(C_BRAM_MAX_ADDR_WIDTH-1 downto 0);
  signal jtag_din_int        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal control_din         : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout        : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0):= (others => '0');
  signal control_dout_int    : std_logic_vector(7 downto 0):= (others => '0');
  signal bram_dout_int       : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0) := (others => '0');
  signal jtag_we_int         : std_logic;
  signal jtag_clk_int        : std_logic;
  signal bram_ce_valid       : std_logic;
  signal din_load            : std_logic;
  --
  signal jtag_dout_0_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_1_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_2_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_3_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_4_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_5_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_6_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal jtag_dout_7_masked  : std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto 0);
  signal picoblaze_reset_int : std_logic_vector(C_NUM_PICOBLAZE-1 downto 0) := (others => '0');
  --        
begin
  bus_zero <= (others => '0');
  --
  jtag_loader_gen: if (C_JTAG_LOADER_ENABLE = 1) generate
    --
    -- Insert BSCAN primitive for target device architecture.
    --
    BSCAN_SPARTAN6_gen: if (C_FAMILY="S6") generate
    begin
      BSCAN_BLOCK_inst : BSCAN_SPARTAN6
      generic map ( JTAG_CHAIN => C_JTAG_CHAIN)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_SPARTAN6_gen;   
    --
    BSCAN_VIRTEX6_gen: if (C_FAMILY="V6") generate
    begin
      BSCAN_BLOCK_inst: BSCAN_VIRTEX6
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => FALSE)
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_VIRTEX6_gen;   
    --
    BSCAN_7SERIES_gen: if (C_FAMILY="7S") generate
    begin
      BSCAN_BLOCK_inst: BSCANE2
      generic map(    JTAG_CHAIN => C_JTAG_CHAIN,
                    DISABLE_JTAG => "FALSE")
      port map( CAPTURE => capture,
                   DRCK => drck,
                  RESET => open,
                RUNTEST => open,
                    SEL => bram_ce_valid,
                  SHIFT => shift,
                    TCK => open,
                    TDI => shift_din,
                    TMS => open,
                 UPDATE => jtag_clk_int,
                    TDO => shift_dout);
    end generate BSCAN_7SERIES_gen;   
    --
    --
    -- Insert clock buffer to ensure reliable shift operations.
    --
    upload_clock: BUFG
    port map( I => drck,
              O => shift_clk);
    --        
    --        
    --  Shift Register      
    --        
    --
    control_reg_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk = '1' then
        if (shift = '1') then
          control_reg_ce <= shift_din;
        end if;
      end if;
    end process control_reg_ce_shift;
    --        
    bram_ce_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          if(C_NUM_PICOBLAZE > 1) then
            for i in 0 to C_NUM_PICOBLAZE-2 loop
              bram_ce(i+1) <= bram_ce(i);
            end loop;
          end if;
          bram_ce(0) <= control_reg_ce;
        end if;
      end if;
    end process bram_ce_shift;
    --        
    bram_we_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          jtag_we_int <= bram_ce(C_NUM_PICOBLAZE-1);
        end if;
      end if;
    end process bram_we_shift;
    --        
    bram_a_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (shift = '1') then
          for i in 0 to C_BRAM_MAX_ADDR_WIDTH-2 loop
            jtag_addr_int(i+1) <= jtag_addr_int(i);
          end loop;
          jtag_addr_int(0) <= jtag_we_int;
        end if;
      end if;
    end process bram_a_shift;
    --        
    bram_d_shift: process (shift_clk)
    begin
      if shift_clk'event and shift_clk='1' then  
        if (din_load = '1') then
          jtag_din_int <= bram_dout_int;
         elsif (shift = '1') then
          for i in 0 to C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-2 loop
            jtag_din_int(i+1) <= jtag_din_int(i);
          end loop;
          jtag_din_int(0) <= jtag_addr_int(C_BRAM_MAX_ADDR_WIDTH-1);
        end if;
      end if;
    end process bram_d_shift;
    --
    shift_dout <= jtag_din_int(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1);
    --
    --
    din_load_select:process (bram_ce, din_load, capture, bus_zero, control_reg_ce) 
    begin
      if ( bram_ce = bus_zero ) then
        din_load <= capture and control_reg_ce;
       else
        din_load <= capture;
      end if;
    end process din_load_select;
    --
    --
    -- Control Registers 
    --
    num_picoblaze <= conv_std_logic_vector(C_NUM_PICOBLAZE-1,3);
    picoblaze_instruction_data_width <= conv_std_logic_vector(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1,5);
    --	
    control_registers: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '0') and (control_reg_ce = '1') then
          case (jtag_addr_int(3 downto 0)) is 
            when "0000" => -- 0 = version - returns (7 downto 4) illustrating number of PB
                           --               and (3 downto 0) picoblaze instruction data width
                           control_dout_int <= num_picoblaze & picoblaze_instruction_data_width;
            when "0001" => -- 1 = PicoBlaze 0 reset / status
                           if (C_NUM_PICOBLAZE >= 1) then 
                            control_dout_int <= picoblaze_reset_int(0) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_0-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0010" => -- 2 = PicoBlaze 1 reset / status
                           if (C_NUM_PICOBLAZE >= 2) then 
                             control_dout_int <= picoblaze_reset_int(1) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_1-1,5) );
                            else 
                             control_dout_int <= (others => '0');
                           end if;
            when "0011" => -- 3 = PicoBlaze 2 reset / status
                           if (C_NUM_PICOBLAZE >= 3) then 
                            control_dout_int <= picoblaze_reset_int(2) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_2-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0100" => -- 4 = PicoBlaze 3 reset / status
                           if (C_NUM_PICOBLAZE >= 4) then 
                            control_dout_int <= picoblaze_reset_int(3) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_3-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0101" => -- 5 = PicoBlaze 4 reset / status
                           if (C_NUM_PICOBLAZE >= 5) then 
                            control_dout_int <= picoblaze_reset_int(4) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_4-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0110" => -- 6 = PicoBlaze 5 reset / status
                           if (C_NUM_PICOBLAZE >= 6) then 
                            control_dout_int <= picoblaze_reset_int(5) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_5-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "0111" => -- 7 = PicoBlaze 6 reset / status
                           if (C_NUM_PICOBLAZE >= 7) then 
                            control_dout_int <= picoblaze_reset_int(6) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_6-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1000" => -- 8 = PicoBlaze 7 reset / status
                           if (C_NUM_PICOBLAZE >= 8) then 
                            control_dout_int <= picoblaze_reset_int(7) & "00" & (conv_std_logic_vector(C_ADDR_WIDTH_7-1,5) );
                           else 
                            control_dout_int <= (others => '0');
                           end if;
            when "1111" => control_dout_int <= conv_std_logic_vector(C_BRAM_MAX_ADDR_WIDTH -1,8);
            when others => control_dout_int <= (others => '1');
          end case;
        else 
          control_dout_int <= (others => '0');
        end if;
      end if;
    end process control_registers;
    -- 
    control_dout(C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-1 downto C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-8) <= control_dout_int;
    --
    pb_reset: process(jtag_clk_int) 
    begin
      if (jtag_clk_int'event and jtag_clk_int = '1') then
        if (bram_ce_valid = '1') and (jtag_we_int = '1') and (control_reg_ce = '1') then
          picoblaze_reset_int(C_NUM_PICOBLAZE-1 downto 0) <= control_din(C_NUM_PICOBLAZE-1 downto 0);
        end if;
      end if;
    end process pb_reset;    
    --
    --
    -- Assignments 
    --
    control_dout (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH-9 downto 0) <= (others => '0') when (C_PICOBLAZE_INSTRUCTION_DATA_WIDTH > 8);
    --
    -- Qualify the blockram CS signal with bscan select output
    jtag_en_int <= bram_ce when bram_ce_valid = '1' else (others => '0');
    --      
    jtag_en_expanded(C_NUM_PICOBLAZE-1 downto 0) <= jtag_en_int;
    jtag_en_expanded(7 downto C_NUM_PICOBLAZE) <= (others => '0') when (C_NUM_PICOBLAZE < 8);
    --        
    bram_dout_int <= control_dout or jtag_dout_0_masked or jtag_dout_1_masked or jtag_dout_2_masked or jtag_dout_3_masked or jtag_dout_4_masked or jtag_dout_5_masked or jtag_dout_6_masked or jtag_dout_7_masked;
    --
    control_din <= jtag_din_int;
    --        
    jtag_dout_0_masked <= jtag_dout_0 when jtag_en_expanded(0) = '1' else (others => '0');
    jtag_dout_1_masked <= jtag_dout_1 when jtag_en_expanded(1) = '1' else (others => '0');
    jtag_dout_2_masked <= jtag_dout_2 when jtag_en_expanded(2) = '1' else (others => '0');
    jtag_dout_3_masked <= jtag_dout_3 when jtag_en_expanded(3) = '1' else (others => '0');
    jtag_dout_4_masked <= jtag_dout_4 when jtag_en_expanded(4) = '1' else (others => '0');
    jtag_dout_5_masked <= jtag_dout_5 when jtag_en_expanded(5) = '1' else (others => '0');
    jtag_dout_6_masked <= jtag_dout_6 when jtag_en_expanded(6) = '1' else (others => '0');
    jtag_dout_7_masked <= jtag_dout_7 when jtag_en_expanded(7) = '1' else (others => '0');
    --
    jtag_en <= jtag_en_int;
    jtag_din <= jtag_din_int;
    jtag_addr <= jtag_addr_int;
    jtag_clk <= jtag_clk_int;
    jtag_we <= jtag_we_int;
    picoblaze_reset <= picoblaze_reset_int;
    --        
  end generate jtag_loader_gen;
--
end Behavioral;
--
--
------------------------------------------------------------------------------------
--
-- END OF FILE m25p16_spi_uart_bridge.vhd
--
------------------------------------------------------------------------------------
